from openopt import NLP
from numpy import cos, arange, ones, asarray, abs, zeros
N = 30
M = 5
ff = lambda x: ((x-M)**2).sum()
p = NLP(ff, cos(arange(N)))

def df(x):
    r = 2*(x-M)
    r[0] += 15 #incorrect derivative
    r[8] += 80 #incorrect derivative
    return r
p.df =  df

p.c = lambda x: [2* x[0] **4-32, x[1]**2+x[2]**2 - 8]

def dc(x):
    r = zeros((2, p.n))
    r[0,0] = 2 * 4 * x[0]**3
    r[1,1] = 2 * x[1]
    r[1,2] = 2 * x[2] + 15 #incorrect derivative
    return r
p.dc = dc

p.h = lambda x: (1e1*(x[-1]-1)**4, (x[-2]-1.5)**4)

def dh(x):
    r = zeros((2, p.n))
    r[0,-1] = 1e1*4*(x[-1]-1)**3
    r[1,-2] = 4*(x[-2]-1.5)**3 + 15 #incorrect derivative
    return r
p.dh = dh

p.checkdf()
p.checkdc()
p.checkdh()
"""
you can use p.checkdF(x) for other point than x0 (F is f, c or h)
p.checkdc(myX)
or
p.checkdc(x=myX)
values with difference greater than
maxViolation (default 1e-5)
will be shown
p.checkdh(maxViolation=1e-4)
p.checkdh(myX, maxViolation=1e-4)
p.checkdh(x=myX, maxViolation=1e-4)

#################################################################################
Typical output (unfortunately, in terminal or other IDEs the blank space used in strings separation can have other lengths):
Note that RD (relative difference) is defined as
int(ceil(log10(abs(Diff) / maxViolation + 1e-150)))
where
Diff = 1 - (info_user+1e-8)/(info_numerical + 1e-8)

OpenOpt checks user-supplied gradient df (shape: (30,) )
according to:
    prob.diffInt = [  1.00000000e-07]
    |1 - info_user/info_numerical| <= prob.maxViolation = 0.01
df num         user-supplied     numerical               RD
    0             +7.000e+00     -8.000e+00              3
    8             -2.291e+00     -1.029e+01              2
max(abs(df_user - df_numerical)) = 14.9999995251
(is registered in df number 0)
========================
OpenOpt checks user-supplied gradient dc (shape: (2, 30) )
according to:
    prob.diffInt = [  1.00000000e-07]
    |1 - info_user/info_numerical| <= prob.maxViolation = 0.01
dc num   i,j:dc[i]/dx[j]   user-supplied     numerical               RD
    32             1 / 2         +1.417e+01     -8.323e-01              4
max(abs(dc_user - dc_numerical)) = 14.9999999032
(is registered in dc number 32)
========================
OpenOpt checks user-supplied gradient dh (shape: (2, 30) )
according to:
    prob.diffInt = [  1.00000000e-07]
    |1 - info_user/info_numerical| <= prob.maxViolation = 0.01
dh num   i,j:dh[i]/dx[j]   user-supplied     numerical               RD
    58            1 / 28         -4.474e+01     -5.974e+01              2
max(abs(dh_user - dh_numerical)) = 14.9999962441
(is registered in dh number 58)
========================

"""



# Example of export OpenOpt MILP to MPS file
# you should have lpsolve and its Python binding properly installed
# (you may take a look at the instructions from openopt.org/LP)

# You can solve problems defined in MPS files 
# with a variety of solvers at NEOS server for free
# http://neos.mcs.anl.gov/
# BTW they have Python API along with web API and other

from numpy import *
from openopt import MILP

f = [1, 2, 3, 4, 5, 4, 2, 1]

# indexing starts from ZERO!
# while in native lpsolve-python wrapper from 1
# so if you used [5,8] for native lp_solve python binding
# you should use [4,7] instead
intVars = [4, 7]

lb = -1.5 * ones(8)
ub = 15 * ones(8)
A = zeros((5, 8))
b = zeros(5)
for i in xrange(5):
    for j in xrange(8):
        A[i,j] = -8+sin(8*i) + cos(15*j)
    b[i] = -150 + 80*sin(80*i)

p = MILP(f=f, lb=lb, ub=ub, A=A, b=b, intVars=intVars)

# if file name not ends with '.MPS' or '.mps'
# then '.mps' will be appended
success = p.exportToMPS('/home/dmitrey/PyTest/milp_1')
# or write into current dir: 
# success = p.exportToMPS('milp')
# success is False if a error occurred (read-only file system, no write access, etc)
# elseware success is True

# f_opt is 25.801450769161505
# x_opt is [ 15. 10.15072538 -1.5 -1.5 -1.  -1.5 -1.5 15.]



"""
Modifying of some solver default parameters is performed via
either kwargs for p.solve() (they can be solver or prob attributes)
or using oosolver (see examples/oosolver.py for more details).
"""
from numpy import *
from openopt import *
f = lambda x: (x[0]-1.5)**2 + sin(0.8 * x[1] ** 2 + 15)**4 + cos(0.8 * x[2] ** 2 + 15)**4 + (x[3]-7.5)**4
lb, ub = -ones(4), ones(4)

# example 1
p = GLP(f, lb=lb, ub=ub,  maxIter = 1e3, maxCPUTime = 3,  maxFunEvals=1e5,  fEnough = 80)
# solve() kwargs can include some prob settings (like maxTime) as well
r = p.solve('galileo', crossoverRate = 0.80, maxTime = 3,  population = 15,  mutationRate = 0.15)

# example 2, via oosolver
solvers = [oosolver('ralg', h0 = 0.80, alp = 2.15, show = False), oosolver('ralg', h0 = 0.15, alp = 2.80, color = 'k')]
for i, solver in enumerate(solvers):
    p = NSP(f, [0]*4, lb=lb, ub=ub, legend='ralg'+str(i+1))
    r = p.solve(solver, plot=True)




#created by Dmitrey

#from numpy import ndarray
#from time import strftime
from threading import Thread
from openopt import __version__ as ooversion
from setDefaultIterFuncs import BUTTON_ENOUGH_HAS_BEEN_PRESSED, USER_DEMAND_EXIT
from ooMisc import killThread
import platform#, pylab

TkinterIsInstalled = True# sometimes Tkinter is not installed
if platform.python_version()[0] == '2': # Python2
    try:
        from Tkinter import Tk, Text, Toplevel, Button, Entry, Menubutton, Label, Frame, \
        StringVar, DISABLED, ACTIVE, Scrollbar
    except:
        TkinterIsInstalled = False
else: # Python3
    try:
        from tkinter import Tk, Text, Toplevel, Button, Entry, Menubutton, Label, Frame, \
        StringVar, DISABLED, ACTIVE, Scrollbar
    except:
        TkinterIsInstalled = False

def manage(p, *args, **kwargs):

    if not TkinterIsInstalled: 
        p.err('''
        Tkinter is not installed. 
        If you have Linux you could try using "apt-get install python-tk"
        ''')
        
    p.__dict__['_immutable'] = False
    bg_color = p._bg_color
    p.isManagerUsed = True
    
    # expected args are (solver, start) or (start, solver) or one of them
    p._args = args
    p._kwargs = kwargs
    

    for arg in args:
        if type(arg) == str or hasattr(arg, '__name__'): 
            p.solver = arg
        elif arg in (0, 1, True, False): 
            start = arg
        else: 
            p.err('Incorrect argument for manage()')

    start = kwargs.pop('start', True)

    if 'solver' in kwargs: 
        p.solver = kwargs['solver']

    # root

    root = Tk()
    p.GUI_root = root
    p.GUI_items = {'root':root}

    '''                                     F0, F_side                                  '''
    useGrid = False
    if useGrid:
        F_side = Frame(root)#, width=30)
        F_side.grid(row=0, column=0, padx=2, pady=2)#, sticky=N+S+E)
        F0 = Frame(root)
        F0.grid(row=0, column=1, padx=2, pady=2)#, sticky=N+S+W)
        F0.pack = F0.grid
        F_side.pack = F_side.grid
        F0.pack_forget = F0.grid_remove
        F_side.pack_forget = F_side.grid_remove
    else:
        F0 = Frame(root)#, width=30)
        F_side = Frame(root)
    F0_side = 'right'
    textSide = 'left'
    p.GUI_items['F_side'] = F_side

    '''                                  F_log, F_result                               '''
    F_log = Frame(F_side)
    F_log.isVisible = False
    p.GUI_items['F_log'] = F_log
    
    F_result = Frame(F_side)
    p.GUI_items['F_result'] = F_result
    F_result.pack(fill='both', expand=True)
    F_result.pack_forget()
    F_result.isVisible = False
    F_result.made = False
    
    
    scrollbar = Scrollbar(F_log)
    scrollbar.pack(side='right', fill='y')
    textOutput = Text(F_log, yscrollcommand=scrollbar.set, bg=bg_color)
    scrollbar.config(command=textOutput.yview)
    
    p._setTextFuncs()
    for func in ('warn', 'err', 'info', 'disp', 'hint', 'pWarn'):
        setattr(p, func, updated_output(getattr(p, func), textOutput))
    
#    if p.managerTextOutput is not False and p.managerTextOutput in ('right', 'left'):
#        textSide = p.managerTextOutput
#        if p.managerTextOutput == 'right': 
#            F0_side = 'left'
    
    textOutput.pack(expand=True, fill='both')
    F_log.isVisible = True
    if p.managerTextOutput is False:
        F_side.pack_forget()
        F_side.isVisible = False
        F_log.isVisible = False

    F_log.pack(fill='both', expand=True)
    
    if not useGrid:
        F_side.pack(side=textSide, fill='both', expand=True)
        F0.pack(side=F0_side, fill='both', expand=True)
        
    F_side.isVisible = True
    F_side.isSideWidget = True
    F_side._root = root
    
    
    """                                               Title                                                """
    #root.wm_title('OpenOpt  ' + ooversion)
    

    """                                              Buttons                                               """

    '''                              Label                          '''
    ButtonsColumn = Frame(F0)
#    F_side._pack_forget = F_side.pack_forget
#    F_side.pack_forget = lambda : f_side_pack_forget(F_side, root, ButtonsColumn)
    ButtonsColumn.pack(ipady=4, expand=True)
    
    Label(ButtonsColumn, text=' OpenOpt ' + ooversion + ' ').pack(expand=True, fill='x')
    Label(ButtonsColumn, text=' Solver: ' + (p.solver if isinstance(p.solver, str) else p.solver.__name__) + ' ').pack(expand=True, fill='x')
    Label(ButtonsColumn, text=' Problem: ' + p.name + ' ').pack(expand=True, fill='x')
    #TODO: mb use Menubutton 

    #Statistics
#    stat = StringVar()
#    stat.set('')
#    Statistics = Button(ButtonsColumn, textvariable = stat, command = lambda: invokeStatistics(p))

#    cw = Entry(ButtonsColumn)
#
#    
#    b = Button(ButtonsColumn, text = 'Evaluate!', command = lambda: invokeCommand(cw))
#    cw.pack(fill='x'', side='left')
#    b.pack(side='right')
        
    '''                              Run                          '''
    
    t = StringVar()
    t.set("      Run      ")
    RunPause = Button(ButtonsColumn, textvariable = t, command = lambda: invokeRunPause(p))
    RunPause.pack(ipady=15, expand=True, fill='x', pady=8)
    p.GUI_items['RunPause'] = RunPause
    p.statusTextVariable = t


    '''                              Log                          '''
        
    Log = Button(ButtonsColumn, text = '    Log    ', command = lambda:invokeLog(p))
    Log.pack(expand=True, fill='x', pady=8, ipady=5)
    p.GUI_items['Log'] = Log

    '''                             Result                         '''
    
    Result = Button(ButtonsColumn, text = '   Result   ', command = lambda: invokeResult(p))
    Result.config(state=DISABLED)
    # TODO: MOP
    if p.probType in ('NLP', 'SNLE', 'NLSP', 'GLP', 'LP', #'MOP',
    'MILP', 'NSP', 'MINLP', 'LLSP', 'SLE', 'TSP', 'SOCP', 'SDP') \
    or p.probType.endswith('QP'):
        Result.pack(expand=True, fill='x', pady=8, ipady=5)
    p.GUI_items['Result'] = Result

    '''                             Enough                         '''

    Enough = Button(ButtonsColumn, text = '   Enough!   ', 
                    command = lambda: invokeEnough(p))
    Enough.config(state=DISABLED)
    Enough.pack(expand=True, fill='x', pady=8, ipady=5)
    p.GUI_items['Enough'] = Enough


    '''                             Exit                         '''

    Quit = Button(ButtonsColumn, text="      Exit      ", command = lambda:invokeExit(p))
    Quit.pack(ipady=15, expand=True, fill='x', pady=8)
    p.GUI_items['Quit'] = Quit
    
    '''                             Time                         '''
    Frame(ButtonsColumn).pack(ipady=0, expand=True, fill='x')
    F = Frame(ButtonsColumn)
    _time = StringVar()
    p._manager_time = 0
    p._manager_time_variable = _time
    Label(F, text=" Time:\t\t").pack(side='left', expand=True, fill='x')
    _time.set("%d" % p._manager_time)
    _Time = Label(F, textvariable = _time)
    _Time.pack(side='right', ipady=1, ipadx=3, padx=3, expand=True, fill='x')
    p.GUI_items['time'] = _time
    F.pack(ipady=1, ipadx=3, padx=3, expand=True, fill='x')
    
    '''                         CPU Time                         '''
    Frame(ButtonsColumn).pack(ipady=0, expand=True, fill='x')
    F = Frame(ButtonsColumn)
    _cputime = StringVar()
    p._manager_cputime = 0
    _cputime.set(p._manager_cputime)
    p._manager_cputime_variable = _cputime
    Label(F, text=" CPU Time:\t").pack(side='left', expand=True, fill='x')
    _time.set("%d" % p._manager_cputime)
    _CPUTime = Label(F, textvariable = _cputime)
    _CPUTime.pack(side='right', ipady=1, ipadx=3, padx=3, expand=True, fill='x')
    p.GUI_items['cputime'] = _cputime
    F.pack(ipady=1, ipadx=3, padx=3, expand=True, fill='x')


    """                                            Start main loop                                      """
    #state = 'paused'

    if start:
        Thread(target=invokeRunPause, args=(p, )).start()
    root.mainloop()
    #finalShow(p)


    """                                              Handle result                                       """

    if hasattr(p, 'tmp_result'):
        r = p.tmp_result
        delattr(p, 'tmp_result')
    else:
        r = None


    """                                                    Return                                           """
    return r
    
########################################################
########################################################
########################################################

def invokeRunPause(p, isEnough=False):
    try:
        import pylab
    except:
        if p.plot: 
            p.warn('to use graphics you should have matplotlib installed')
            p.plot = False
        
    if isEnough:
        p.GUI_items['RunPause'].config(state=DISABLED)

    if p.state == 'init':
        p.probThread = Thread(target=doCalculations, args=(p, ))
        p.state = 'running'
        p.statusTextVariable.set('    Pause    ')
        p.GUI_items['Enough'].config(state=ACTIVE)
        p.GUI_root.update_idletasks()
        p.probThread.start()

    elif p.state == 'running':
        p.state = 'paused'
        if p.plot: 
            pylab.ioff()
        p.statusTextVariable.set('      Run      ')
        p.GUI_root.update_idletasks()

    elif p.state == 'paused':
        p.state = 'running'
        if p.plot:
            pylab.ion()
        p.statusTextVariable.set('    Pause    ')
        p.GUI_root.update_idletasks()
        
########################################################
def doCalculations(p):
    p._immutable = False
    try:
        p.__dict__['tmp_result'] = p.solve(*p._args, **p._kwargs)
    except killThread:
        if p.plot and hasattr(p, 'figure'):
            #p.figure.canvas.draw_drawable = lambda: None
            try:
                import pylab
                pylab.ioff()
                pylab.close('all')
            except:
                pass
#    p._immutable = True
    p.GUI_items['Result'].config(state='active')
    p.GUI_items['RunPause'].config(state=DISABLED)
    p.GUI_items['Enough'].config(state=DISABLED)
#    p.GUI_items['Quit'].config(state='active')
        

########################################################
#def invokeStatistics(p):
def invokeCommand(cw):
    exec(cw.get())
    
########################################################
def updated_output(func, textOutput):
    def func2(msg):
        msgCap = func.s 
        Set = getattr(func, 'set', ())
        if msg in Set:
            return
        if msgCap == 'OpenOpt Error: ': #if type(Set) != tuple:
            textOutput.tag_config(msg, foreground="red")
            textOutput.config(foreground="red")
        textOutput.insert('end', msgCap + msg + '\n')
        textOutput.yview('end')
        func(msg)
    return func2
    
########################################################
def invokeResult(p):
    res = w = p.GUI_items['F_result']
    log = p.GUI_items['F_log']
    side = p.GUI_items['F_side']
    
#    p.disp('%s %s' % (side.isVisible, res.isVisible))
    r = p.tmp_result
    
    r._tkview(w)
        
    if side.isVisible:
        if res.isVisible:
            withdraw(res, side, root=p.GUI_items['root'])
        else:
            restore(res)
    else:
        restore(side, res)
    withdraw(log)
    
########################################################
def invokeExit(p):
    p._immutable = False
    p.userStop = True
    p.istop = USER_DEMAND_EXIT
    if hasattr(p, 'stopdict'): 
        p.stopdict[USER_DEMAND_EXIT] = True

    # however, the message is currently unused
    # since openopt return r = None
    p.msg = 'user pressed Exit button'

    p.GUI_root.destroy()

########################################################
def invokeLog(p):
    log = p.GUI_items['F_log']
    res = p.GUI_items['F_result']
    side = p.GUI_items['F_side']
    
    withdraw(res)
    if side.isVisible:
        if log.isVisible:
            withdraw(side, root=p.GUI_items['root'])
        else:
            restore(log)
    else:
        restore(side, log)
        
########################################################
def invokeEnough(p):
    
    p.userStop = True
    p.istop = BUTTON_ENOUGH_HAS_BEEN_PRESSED
    if hasattr(p, 'stopdict'):  
        p.stopdict[BUTTON_ENOUGH_HAS_BEEN_PRESSED] = True
    p.msg = 'button Enough has been pressed'

    if p.state == 'paused':
        invokeRunPause(p, isEnough=True)
    else:
        p.GUI_items['RunPause'].config(state=DISABLED)
        p.GUI_items['Enough'].config(state=DISABLED)
        p.GUI_items['Quit'].config(state=DISABLED)
        

#TODO: simplify it
########################################################
def restore(*args):
    for w in args:
        status = w.isVisible
#        assert w.winfo_viewable() == w.isVisible
        if status is False:
            w.pack(fill='both', expand=True)
            isSideWidget = getattr(w, 'isSideWidget', False)
            if isSideWidget:
                tmp = w._root.geometry() 
                sizes, x, y = tmp.split('+')
                size_x, size_y = sizes.split('x')
                X, Y = w.xy_sizes
                w._root.geometry('%dx%d+%d+%d'%(X+int(size_x), Y, int(x)-X, int(y)))
            w.isVisible = True
    
        
########################################################
def withdraw(*args, **kw):
    for w in args:
        status = w.isVisible
#        assert w.winfo_viewable() == w.isVisible
        if status:
            isSideWidget = getattr(w, 'isSideWidget', False)
            if isSideWidget:
                root=kw['root']
                width = w.winfo_width()
                tmp = root.geometry() 
                sizes, x, y = tmp.split('+')
                size_x, size_y = sizes.split('x')
                w.xy_sizes = (w.winfo_width(), w.winfo_height())
                sizes = str(int(size_x)-width) + 'x' + size_y
                w.pack_forget()
                new = sizes + '+'+str(int(x)+width)+'+'+y
                root.geometry(new)
            else:
                w.pack_forget()
            w.isVisible = False



from openopt import GLP
from numpy import *
N = 100
aN = arange(N)
f = lambda x: ((x-aN)**2).sum()
p = GLP(f, lb = -ones(N),  ub = N*ones(N),  maxIter = 1e3,  maxFunEvals = 1e5,  maxTime = 10,  maxCPUTime = 300)

#optional: graphic output
#p.plot = 1

r = p.solve('de', plot=1, debug=1, iprint=0)
x_opt,  f_opt = r.xf,  r.ff



from openopt.kernel.baseSolver import baseSolver
from QPSolve import QPSolve

class qlcp(baseSolver):
    __license__ = "MIT"
    __authors__ = "Enzo Michelangeli"
    #_requiresBestPointDetection = True
    
    __name__ = 'qlcp'
    __alg__ = 'Lemke algorithm, using linear complementarity problem'
    #__isIterPointAlwaysFeasible__ = True
    
    __optionalDataThatCanBeHandled__ = ['lb', 'ub', 'A', 'b', 'Aeq', 'beq']
    
    def __init__(self): pass
    def __solver__(self, p):
        # TODO: add QI
        x,  retcode = QPSolve(p.H, p.f, p.A, p.b, p.Aeq, p.beq, p.lb, p.ub)
#Q, e, A=None, b=None, Aeq=None, beq=None, lb=None, ub=None, QI=None
        if retcode[0] == 1:
            p.istop = 1000
            p.xf = x
        else:
            p.istop = -1

        # TODO: istop, msg wrt retcode
        



from openopt.kernel.baseSolver import *
from openopt.kernel.setDefaultIterFuncs import SMALL_DELTA_X,  SMALL_DELTA_F
from wh import wh

def FuncWrapper(p, attr):
    Func = getattr(p, attr)
    func = lambda *args : Func(*(arg.flatten() for arg in args))
    return func

class wh_conn_solver(baseSolver):
    __license__ = "proprietary"
    __authors__ = ""
    __alg__ = ""
    iterfcnConnected = True
    _canHandleScipySparse = True
    matlab = 'matlab'
    arrAttribs = ()
    funcAttribs = ()

    def __init__(self): 
        pass
    def __solver__(self, p):
        # TODO: cons patterns
#        if p.nc != 0: r.append(p._getPattern(p.user.c))
#        if p.nh != 0: r.append(p._getPattern(p.user.h))
        p.kernelIterFuncs.pop(SMALL_DELTA_X, None)
        p.kernelIterFuncs.pop(SMALL_DELTA_F, None)
        
        Data = {
        'p': p, 
        'TolFun': p.ftol, 
        'RelTol': getattr(p, 'reltol', None),  # used in ODE
        'AbsTol': getattr(p, 'abstol', None),  # used in ODE
        'TolCon': p.contol, 
        'TolX': p.xtol, 
        'solver_id': self.solver_id
        }
        for attr in self.arrAttribs:
            Data[attr] = getattr(p, attr)
        for attr in self.funcAttribs:
            Data[attr] = FuncWrapper(p, attr)#getattr(p, attr)(x.flatten())

        wh(Data, self.matlab)
        if p.istop == 0:
            p.istop = 1000




from numpy import empty, sin, cos, arange, ones
from openopt import LLAVP

M, N = 150, 15
C = empty((M,N))
d =  empty(M)

for j in range(M):
    d[j] = 1.5*N+80*sin(j)
    C[j] = 8*sin(4.0+arange(N)) + 15*cos(j)

lb = sin(arange(N))
ub = lb + 1
p = LLAVP(C, d, lb=lb, ub=ub, dump = 10,  X = ones(N),  maxIter = 1e4, maxFunEvals = 1e100)

#optional: plot
p.plot=1

r = p.solve('nsp:ralg', iprint = 100, maxIter = 1000)
#r = p.solve('nsp:ipopt', iprint = 100, maxIter = 1000)


print('f_opt: %f' % r.ff)
#print 'x_opt:', r.xf




"""
Example:
Let's concider the problem
15x1 + 8x2 + 80x3 -> min        (1)
subjected to
x1 + 2x2 + 3x3 <= 15              (2)
8x1 +  15x2 +  80x3 <= 80      (3)
8x1  + 80x2 + 15x3 <=150      (4)
100x1 +  10x2 + x3 >= 800     (5)
80x1 + 8x2 + 15x3 = 750         (6)
x1 + 10x2 + 100x3 = 80           (7)
x1 >= 4                                     (8)
-8 >= x2 >= -80                        (9)
"""

from numpy import *
from openopt import LP
f = array([15,8,80])
A = mat('1 2 3; 8 15 80; 8 80 15; -100 -10 -1') # numpy.ndarray is also allowed
b = [15, 80, 150, -800] # numpy.ndarray, matrix etc are also allowed
Aeq = mat('80 8 15; 1 10 100') # numpy.ndarray is also allowed
beq = (750, 80)

lb = [4, -80, -inf]
ub = [inf, -8, inf]
p = LP(f, A=A, Aeq=Aeq, b=b, beq=beq, lb=lb, ub=ub)
#or p = LP(f=f, A=A, Aeq=Aeq, b=b, beq=beq, lb=lb, ub=ub)

#r = p.minimize('glpk') # CVXOPT must be installed
#r = p.minimize('lpSolve') # lpsolve must be installed
r = p.minimize('pclp') 
#search for max: r = p.maximize('glpk') # CVXOPT & glpk must be installed
#r = p.minimize('nlp:ralg', ftol=1e-7, xtol=1e-7, goal='min', plot=1) 

print('objFunValue: %f' % r.ff) # should print 204.48841578
print('x_opt: %s' % r.xf) # should print [ 9.89355041 -8.          1.5010645 ]



from numpy import dot, zeros, float64, diag, ones, ndarray, isfinite, all, asarray
from numpy.linalg import norm

class Dilation():
    def __init__(self, p): 
        self.T = float64
#        if hasattr(numpy, 'float128'):
#            pass
#            self.T = numpy.float128        
        self.b = diag(ones(p.n, dtype = self.T))
        
    def getDilatedVector(self, vec):
        assert type(vec) == ndarray and all(isfinite(vec))
        tmp = dot(self.b.T, vec)
        if any(tmp): tmp /= norm(tmp)
        return dot(self.b, tmp)
    
    def updateDilationMatrix(self, vec, alp=2.0):
        assert type(vec) == ndarray and all(isfinite(vec))
        g = dot(self.b.T, vec)
        ng = norm(g)

#        if self.needRej(p, b, g1, g) or selfNeedRej:
#            selfNeedRej = False
#            if self.showRej or p.debug:
#                p.info('debug msg: matrix B restoration in ralg solver')
#            b = B0.copy()
#            hs = p.norm(prevIter_best_ls_point.x - best_ls_point.x)
#            # TODO: iterPoint = projection(iterPoint,Aeq) if res_Aeq > 0.75*contol

#        if ng < 1e-40: 
#            hs *= 0.9
#            p.debugmsg('small dilation direction norm (%e), skipping' % ng)

        if all(isfinite(g)) and ng > 1e-50:
            g = (g / ng).reshape(-1,1)
            vec1 = dot(self.b, g)
            w = asarray(1.0/alp-1.0, self.T) 
#            w = asarray(1.0/(alp+alp_addition)-1.0, T) 
            vec2 = w * g.T
            self.b += dot(vec1, vec2)
            



import os,sys
sys.path.append(os.getcwd()+os.sep+'kernel')

from LP import LP as CLP
from LCP import LCP as CLCP
from EIG import EIG as CEIG
from SDP import SDP as CSDP
from QP import QP as CQP
from MILP import MILP as CMILP
from STAB import STAB as CSTAB
from MCP import MCP as CMCP
from DSP import DSP as CDSP
from TSP import TSP as CTSP
from KSP import KSP as CKSP
from BPP import BPP as CBPP
from NSP import NSP as CNSP
from NLP import NLP as CNLP
from MOP import MOP as CMOP
from MINLP import MINLP as CMINLP
from NLSP import NLSP as CNLSP
from NLLSP import NLLSP as CNLLSP
from GLP import GLP as CGLP
from SLE import SLE as CSLE
from LLSP import LLSP as CLLSP
from MMP import MMP as CMMP
from LLAVP import LLAVP as CLLAVP
from LUNP import LUNP as CLUNP
from SOCP import SOCP as CSOCP
from DFP import DFP as CDFP
from IP import IP as CIP
from ODE import ODE as CODE


def MILP(*args, **kwargs):
    """
    MILP: constructor for Mixed Integer Linear Problem assignment
    f' x -> min
    subjected to
    lb <= x <= ub
    A x <= b
    Aeq x = beq
    for all i from intVars: i-th coordinate of x is required to be integer
    for all j from binVars: j-th coordinate of x is required to be from {0, 1}

    Examples of valid calls:
    p = MILP(f, <params as kwargs>)
    p = MILP(f=objFunVector, <params as kwargs>)
    p = MILP(f, A=A, intVars = myIntVars, Aeq=Aeq, b=b, beq=beq, lb=lb, ub=ub, binVars = binVars)
    See also: /examples/milp_*.py

    :Parameters:
    - intVars : Python list of those coordinates that are required to be integers.
    - binVars : Python list of those coordinates that are required to be binary.
    all other input parameters are same to LP class constructor ones

    :Returns:
    OpenOpt MILP class instance

    Notes
    -----
    Solving of MILPs is performed via
    r = p.solve(string_name_of_solver) 
    or  p.maximize, p.minimize
    r.xf - desired solution (NaNs if a problem occured)
    r.ff - objFun value (<f,x_opt>) (NaN if a problem occured)
    (see also other r fields)
    Solvers available for now:
    lpSolve (LGPL) - requires lpsolve + Python bindings installations (all mentioned is available in http://sourceforge.net/projects/lpsolve)
    glpk (GPL 2) - requires glpk + CVXOPT v >= 1.0 installations (read OO MILP webpage for more details)
    """
    return CMILP(*args, **kwargs)

def LP(*args, **kwargs):
    """
    LP: constructor for Linear Problem assignment
    f' x -> min
    subjected to
    lb <= x <= ub
    A x <= b
    Aeq x = beq

    valid calls are:
    p = LP(f, <params as kwargs>)
    p = LP(f=objFunVector, <params as kwargs>)
    p = LP(f, A=A, Aeq=Aeq, Awhole=Awhole, b=b, beq=beq, bwhole=bwhole, dwhole=dwhole, lb=lb, ub=ub)
    See also: /examples/lp_*.py

    :Parameters:
    f: vector of length n
    A: size m1 x n matrix, subjected to A * x <= b
    Aeq: size m2 x n matrix, subjected to Aeq * x = beq
    b, beq: corresponding vectors of lengthes m1, m2
    lb, ub: vectors of length n, some coords may be +/- inf

    :Returns:
    OpenOpt LP class instance

    Notes
    -----
    Solving of LPs is performed via
    r = p.solve(string_name_of_solver)
    or  p.maximize, p.minimize
    r.xf - desired solution (NaNs if a problem occured)
    r.ff - objFun value (<f,x_opt>) (NaN if a problem occured)
    (see also other r fields)
    Solvers available for now:
    pclp (BSD) - premature but pure Python implementation with permissive license
    lpSolve (LGPL) - requires lpsolve + Python bindings installations (all mentioned is available in http://sourceforge.net/projects/lpsolve)
    cvxopt_lp (GPL) - requires CVXOPT (http://abel.ee.ucla.edu/cvxopt)
    glpk(GPL2) - requires CVXOPT(http://abel.ee.ucla.edu/cvxopt) & glpk (www.gnu.org/software/glpk)
    converter to NLP. Example: r = p.solve('nlp:ipopt')
    """
    return CLP(*args, **kwargs)

def STAB(*args, **kwargs):
    """
    STAB: constructor for graph stability number problem assignment
    
    valid calls are:
    p = STAB(graph, <params as kwargs>)
    p = STAB(graph = graph, <params as kwargs>)
    See also: /examples/stab_*.py

    :Parameters:
    graph: networkx graph instance

    :Returns:
    OpenOpt STAB class instance

    Notes
    -----
    Solving of OpenOpt STAB problems is performed via
    r = p.solve(string_name_of_solver)
    or r = p.manage(string_name_of_solver) (to enable basic GUI)
    r.solution - desired solution (python list of nodes)
    r.ff - objFun value (NaN if a problem occured)
    (see also other r fields)
    Solvers available for now:  
       * interalg (BSD license)
       * MILP solvers at http://openopt.org/MILP
    """
    return CSTAB(*args, **kwargs)


def MCP(*args, **kwargs):
    """
    MCP: constructor for maximum clique problem assignment
    
    valid calls are:
    p = MCP(graph, <params as kwargs>)
    p = MCP(graph = graph, <params as kwargs>)
    See also: /examples/mcp_*.py

    :Parameters:
    graph: networkx graph instance

    :Returns:
    OpenOpt MCP class instance

    Notes
    -----
    Solving of OpenOpt MCP problems is performed via
    r = p.solve(string_name_of_solver)
    or r = p.manage(string_name_of_solver) (to enable basic GUI)
    r.solution - desired solution (python list of nodes)
    r.ff - objFun value (NaN if a problem occured)
    (see also other r fields)
    Solvers available for now:  
       * interalg (BSD license)
       * MILP solvers at http://openopt.org/MILP
    """
    return CMCP(*args, **kwargs)

def DSP(*args, **kwargs):
    """
    DSP: constructor for domminating set problem assignment
    
    valid calls are:
    p = DSP(graph, <params as kwargs>)
    p = DSP(graph = graph, <params as kwargs>)
    See also: /examples/mcp_*.py

    :Parameters:
    graph: networkx graph instance

    :Returns:
    OpenOpt DSP class instance

    Notes
    -----
    Solving of OpenOpt DSP problems is performed via
    r = p.solve(string_name_of_solver)
    or r = p.manage(string_name_of_solver) (to enable basic GUI)
    r.solution - desired solution (python list of nodes)
    r.ff - objFun value (NaN if a problem occured)
    (see also other r fields)
    Solvers available for now:  
       * interalg (BSD license)
       * MILP solvers at http://openopt.org/MILP
    """
    return CDSP(*args, **kwargs)
    
def TSP(*args, **kwargs):
    """
    TSP: constructor for traveling salesman problem assignment
    
    valid calls are:
    p = TSP(graph, <params as kwargs>)
    p = TSP(graph = graph, <params as kwargs>)
    See also: /examples/tsp_*.py

    :Parameters:
    graph: networkx graph instance

    :Returns:
    OpenOpt TSP class instance

    Notes
    -----
    Solving of OpenOpt TSP problems is performed via
    r = p.solve(string_name_of_solver)
    or r = p.manage(string_name_of_solver) (to enable basic GUI)
    r.solution - desired solution (python list of edges)
    r.ff - objFun value (NaN if a problem occured)
    (see also other r fields)
    Solvers available for now:  
       * interalg (BSD license)
       * MILP solvers at http://openopt.org/MILP
    """
    return CTSP(*args, **kwargs)


def KSP(*args, **kwargs):
    """
    KSP: constructor for knapsack problem assignment
    
    valid calls are:
    p = KSP(objective, objects, <params as kwargs>)
    See also: /examples/ksp_*.py

    :Parameters:
    objective: objective (e.g. 'weight', 'cost')
    objects: Python list or tuple of dictionaries

    :Returns:
    OpenOpt KSP class instance

    Notes
    -----
    Solving of OpenOpt KSP problems is performed via
    r = p.solve(string_name_of_solver)
    or r = p.manage(string_name_of_solver) (to enable basic GUI)
    r.solution - desired solution (names or numbers of objects)
    r.ff - objFun value (NaN if a problem occured)
    (see also other r fields)
    Solvers available for now:  
       * MILP solvers at http://openopt.org/MILP
       * interalg (http://openopt.org/interalg)
    """
    return CKSP(*args, **kwargs)

def BPP(*args, **kwargs):
    """
    BPP: constructor for traveling salesman problem assignment
    
    valid calls are:
    p = BPP(items, <params as kwargs>)
    See also: /examples/bpp_*.py

    :Parameters:
    objective: objective (e.g. 'weight', 'cost')
    items: Python list or tuple of dictionaries

    :Returns:
    OpenOpt BPP class instance

    Notes
    -----
    Solving of OpenOpt BPP problems is performed via
    r = p.solve(string_name_of_solver)
    or r = p.manage(string_name_of_solver) (to enable basic GUI)
    r.solution - desired solution (names or numbers of objects)
    r.ff - objFun value (NaN if a problem occured)
    (see also other r fields)
    Solvers available for now:  
       * MILP solvers at http://openopt.org/MILP
    """
    return CBPP(*args, **kwargs)

def LCP(*args, **kwargs):
    """
    LCP: constructor for Linear Complementarity Problem assignment
    find w, z: w = Mz + q
    
    valid calls are:
    p = LCP(M, q, <params as kwargs>)
    p = LCP(M=M, q=q, <params as kwargs>)
    See also: /examples/lcp_*.py

    :Parameters:
    M: numpy array of size n x n
    q: vector of length n

    :Returns:
    OpenOpt LCP class instance

    Notes
    -----
    Solving of LCPs is performed via
    r = p.solve(string_name_of_solver)
    r.xf - desired solution (1st n/2 coords are w, other n/2 coords are z)
    r.ff - objFun value (max residual of Mz+q-w) (NaN if a problem occured)
    (see also other r fields)
    Solvers available for now:
    lcpsolve (BSD) 

    """
    return CLCP(*args, **kwargs)

def EIG(*args, **kwargs):
    """
    EIG: constructor for Eigenvalues Problem assignment
    to solve standard eigenvalue problem:
        find eigenvalues and eigenvectors of square matrix A:   
            A x = lambda x
    or general eigenvalue problem:
        A x = lambda M x
    
    valid calls are:
    p = EIG(M, q, <params as kwargs>)
    p = EIG(M=M, q=q, <params as kwargs>)
    See also: /examples/eig_*.py

    :Parameters:
    A, (optional) M: numpy array or scipy sparse matrix of size n x n

    :Returns:
    OpenOpt EIG class instance

    Notes
    -----
    Solving of EIGs is performed via
    r = p.solve(string_name_of_solver)
    see http://openopt.org/EIG for more info
    Solvers available for now:
    arpack (license: BSD) 

    """
    return CEIG(*args, **kwargs)

def SDP(*args, **kwargs):
    """
    SDP: constructor for SemiDefinite Problem assignment
    f' x -> min
    subjected to
    lb <= x <= ub
    A x <= b
    Aeq x = beq
    For all i = 0, ..., I: Sum [j = 0, ..., n-1] {S_i_j x_j} <= d_i (matrix componentwise inequality),
    d_i are square matrices 
    S_i_j are square positive semidefinite matrices of size same to d_i
    

    valid calls are:
    p = SDP(f, <params as kwargs>)
    p = SDP(f=objFunVector, <params as kwargs>)
    p = SDP(f, S=S, d=d, A=A, Aeq=Aeq, Awhole=Awhole, b=b, beq=beq, bwhole=bwhole, dwhole=dwhole, lb=lb, ub=ub)

    See also: /examples/sdp_*.py

    :Parameters:
    f: vector of length n
    S: Python dict of square matrices S[0, 0], S[0,1], ..., S[I,J]
    S[i, j] are real symmetric positive-definite matrices
    d: Python dict of square matrices d[0], ..., d[I]
    A: size m1 x n matrix, subjected to A * x <= b
    Aeq: size m2 x n matrix, subjected to Aeq * x = beq
    b, beq: corresponding vectors of lengthes m1, m2
    lb, ub: vectors of length n, some coords may be +/- inf

    :Returns:
    OpenOpt SDP class instance

    Notes
    -----
    Solving of SDPs is performed via
    r = p.solve(string_name_of_solver)
    r.xf - desired solution (NaNs if a problem occured)
    r.ff - objFun value (<f,x_opt>) (NaN if a problem occured)
    (see also other r fields)
    Solvers available for now:
    cvxopt_sdp (LGPL) - requires CVXOPT (http://abel.ee.ucla.edu/cvxopt)
    dsdp (GPL) - requires CVXOPT + DSDP installation, can't handle linear equality constraints Aeq x = beq
    
    """
    return CSDP(*args, **kwargs)

def SOCP(*args, **kwargs):
    """
    SOCP: constructor for Second-Order Cone Problem assignment
    f' x -> min
    subjected to
    lb <= x <= ub
    Aeq x = beq
    For all i = 0, ..., I: ||C_i x + d_i|| <= q_i x + s_i    

    valid calls are:
    p = SDP(f, <params as kwargs>)
    p = SDP(f=objFunVector, <params as kwargs>)
    p = SDP(f, S=S, d=d, A=A, Aeq=Aeq, Awhole=Awhole, b=b, beq=beq, bwhole=bwhole, dwhole=dwhole, lb=lb, ub=ub)

    See also: /examples/sdp_*.py

    :Parameters:
    f: vector of length n
    Aeq: size M x n matrix, subjected to Aeq * x = beq
    beq: corresponding vector of length M
    C: Python list of matrices C_i of shape (m_i, n)
    d: Python list of vectors of length m_i
    q: Python list of vectors of length n
    s: Python list of numbers, len(s) = n
    
    lb, ub: vectors of length n, some coords may be +/- inf

    :Returns:
    OpenOpt SDP class instance

    Notes
    -----
    Solving of SOCPs is performed via
    r = p.solve(string_name_of_solver)
    r.xf - desired solution (NaNs if a problem occured)
    r.ff - objFun value (<f,x_opt>) (NaN if a problem occured)
    (see also other r fields)
    Solvers available for now:
    cvxopt_socp (LGPL) - requires CVXOPT (http://abel.ee.ucla.edu/cvxopt)
    
    """
    return CSOCP(*args, **kwargs)

def QP(*args, **kwargs):
    """
    QP: constructor for Quadratic Problem assignment
    1/2 x' H x  + f' x -> min
    subjected to
    A x <= b
    Aeq x = beq
    lb <= x <= ub

    Examples of valid calls:
    p = QP(H, f, <params as kwargs>)
    p = QP(numpy.ones((3,3)), f=numpy.array([1,2,4]), <params as kwargs>)
    p = QP(f=range(8)+15, H = numpy.diag(numpy.ones(8)), <params as kwargs>)
    p = QP(H, f, A=A, Aeq=Aeq, b=b, beq=beq, lb=lb, ub=ub, <other params as kwargs>)
    See also: /examples/qp_*.py

    INPUT:
    H: size n x n matrix, symmetric, positive-definite
    f: vector of length n
    lb, ub: vectors of length n, some coords may be +/- inf
    A: size m1 x n matrix, subjected to A * x <= b
    Aeq: size m2 x n matrix, subjected to Aeq * x = beq
    b, beq: vectors of lengths m1, m2
    Alternatively to A/Aeq you can use Awhole matrix as it's described in LP documentation (or both A, Aeq, Awhole)
    OUTPUT: OpenOpt QP class instance

    Solving of QPs is performed via
    r = p.solve(string_name_of_solver)
    r.xf - desired solution (NaNs if a problem occured)
    r.ff - objFun value (NaN if a problem occured)
    (see also other r fields)
    Solvers available for now:
    cvxopt_qp (GPL) - requires CVXOPT (http://abel.ee.ucla.edu/cvxopt)
    converter to NLP. Example: r = p.solve('nlp:ipopt')
    """
    return CQP(*args, **kwargs)

def NLP(*args, **kwargs):
    """
    NLP: constructor for general Non-Linear Problem assignment

    f(x) -> min (or -> max)
    subjected to
    c(x) <= 0
    h(x) = 0
    A x <= b
    Aeq x = beq
    lb <= x <= ub

    Examples of valid usage:
    p = NLP(f, x0, <params as kwargs>)
    p = NLP(f=objFun, x0 = myX0, <params as kwargs>)
    p = NLP(f, x0, A=A, df = objFunGradient, Aeq=Aeq, b=b, beq=beq, lb=lb, ub=ub)
    See also: /examples/nlp_*.py

    INPUTS:
    f: objFun
    x0: start point, vector of length n

    Optional:
    name: problem name (string), is used in text & graphics output
    df: user-supplied gradient of objective function
    c, h - functions defining nonlinear equality/inequality constraints
    dc, dh - functions defining 1st derivatives of non-linear constraints

    A: size m1 x n matrix, subjected to A * x <= b
    Aeq: size m2 x n matrix, subjected to Aeq * x = beq
    b, beq: corresponding vectors of lengthes m1, m2
    lb, ub: vectors of length n subjected to lb <= x <= ub constraints, may include +/- inf values

    iprint = {10}: print text output every <iprint> iteration
    goal = {'minimum'} | 'min' | 'maximum' | 'max' - minimize or maximize objective function
    diffInt = {1e-7} : finite-difference gradient aproximation step, scalar or vector of length nVars
    scale = {None} : scale factor, see /examples/badlyScaled.py for more details
    stencil = {1}|2|3: finite-differences derivatives approximation stencil, 
    used by most of solvers (except scipy_cobyla) when no user-supplied for 
    objfun / nonline constraints derivatives are provided
        1: (f(x+dx)-f(x))/dx (faster but less precize)
        2: (f(x+dx)-f(x-dx))/(2*dx) (slower but more exact)
        3: (-f(x+2*dx)+8*f(x+dx)-8*f(x-dx)+f(x-2*dx))/(12*dx) (even more slower, but even more exact)
    check.df, check.dc, check.dh: if set to True, OpenOpt will check user-supplied gradients.
    args (or args.f, args.c, args.h) - additional arguments to objFunc and non-linear constraints,
        see /examples/userArgs.py for more details.

    contol: max allowed residual in optim point
    (for any constraint from problem constraints:
    constraint(x_optim) < contol is required from solver)

    stop criteria:
    maxIter {400}
    maxFunEvals {1e5}
    maxCPUTime {inf}
    maxTime {inf}
    maxLineSearch {500}
    fEnough {-inf for min problems, +inf for max problems}:
        stop if objFunc vulue better than fEnough and all constraints less than contol
    ftol {1e-6}: used in stop criterium || f[iter_k] - f[iter_k+1] || < ftol
    xtol {1e-6}: used in stop criterium || x[iter_k] - x[iter_k+1] || < xtol
    gtol {1e-6}: used in stop criteria || gradient(x[iter_k]) || < gtol

    callback - user-defined callback function(s), see /examples/userCallback.py

    Notes:
    1) for more safety default values checking/reassigning (via print p.maxIter / prob.maxIter = 400) is recommended
    (they may change in future OpenOpt versions and/or not updated in time in the documentation)
    2) some solvers may ignore some of the stop criteria above and/or use their own ones
    3) for NSP constructor ftol, xtol, gtol defaults may have other values

    graphic options:
    plot = {False} | True : plot figure (now implemented for UC problems only), requires matplotlib installed
    color = {'blue'} | black | ... (any valid matplotlib color)
    specifier = {'-'} | '--' | ':' | '-.' - plot specifier
    show = {True} | False : call pylab.show() after solver finish or not
    xlim {(nan, nan)}, ylim {(nan, nan)} - initial estimation for graphical output borders
    (you can use for example p.xlim = (nan, 10) or p.ylim = [-8, 15] or p.xlim=[inf, 15], only real finite values will be taken into account)
    for constrained problems ylim affects only 1st subplot
    p.graphics.xlabel or p.xlabel = {'time'} | 'cputime' | 'iter' # desired graphic output units in x-axe, case-unsensetive


    Note: some Python IDEs have problems with matplotlib!

    Also, after assignment NLP instance you may modify prob fields inplace:
    p.maxIter = 1000
    p.df = lambda x: cos(x)

    OUTPUT: OpenOpt NLP class instance

    Solving of NLPs is performed via
    r = p.solve(string_name_of_solver)
    or  p.maximize, p.minimize
    r.xf - desired solution (NaNs if a problem occured)
    r.ff - objFun value (NaN if a problem occured)
    (see also other fields, such as CPUTimeElapsed, TimeElapsed, isFeasible, iter etc, via dir(r))

    Solvers available for now:
    single-variable:
        goldenSection, scipy_fminbound (latter is not recommended)
        (both these solvers require finite lb-ub and ignore user-supplied gradient)
    unconstrained:
        scipy_bfgs, scipy_cg, scipy_ncg, 
        (these ones cannot handle user-provided gradient) scipy_powell and scipy_fmin 
        amsg2p - requires knowing fOpt (optimal value)
    box-bounded:
        scipy_lbfgsb, scipy_tnc - require scipy installed
        bobyqa - doesn't use derivatives; requires http://openopt.org/nlopt installed
        ptn, slmvm1, slmvm2 - require http://openopt.org/nlopt installed
    all constraints:
        ralg
        ipopt (requires ipopt + pyipopt installed)
        scipy_slsqp
        scipy_cobyla (this one cannot handle user-supplied gradients)
        lincher (requires CVXOPT QP solver),
        gsubg - for large-scaled problems
        algencan (ver. 2.0.3 or more recent, very powerful constrained solver, GPL,
        requires ALGENCAN + Python interface installed,
        see http://www.ime.usp.br/~egbirgin/tango/)
        mma and auglag - require http://openopt.org/nlopt installed

    """
    return CNLP(*args, **kwargs)

def MINLP(*args, **kwargs):
    """
    MINLP: constructor for general Mixed-Integer Non-Linear Problem assignment
    parameters and usage: same to NLP, + parameters
    discreteVars: dictionary numberOfCoord <-> list (or tuple) of allowed values, eg
        p.discreteVars = {0: [1, 2.5], 15: (3.1, 4), 150: [4,5, 6]}
    discrtol (default 1e-5) - tolerance required for discrete constraints 
    available solvers: 
    branb (branch-and-bound) - translation of fminconset routine, requires non-default string parameter nlpSolver
    """
    return CMINLP(*args, **kwargs)

def NSP(*args, **kwargs):
    """
    Non-Smooth Problem constructor
    Same usage as NLP (see help(NLP) and /examples/nsp_*.py), but default values of contol, xtol, ftol, diffInt may differ
    Also, default finite-differences derivatives approximation stencil is 3 instead of 1 for NLP
    Solvers available for now:
        ralg - all constraints, medium-scaled (nVars = 1...1000), can handle user-provided gradient/subgradient
        amsg2p - requires knowing fOpt (optimal value), medium-scaled (nVars = 1...1000), can handle user-provided gradient/subgradient
        gsubg - for large-scaled problems
        scipy_fmin - a Nelder-Mead simplex algorithm implementation, cannot handle constraints and derivatives
        sbplx  -  A variant of Nelder-Mead algorithm; requires http://openopt.org/nlopt installed
        ShorEllipsoid (unconstrained for now) - small-scale, nVars=1...10, requires r0: ||x0-x*||<=r0
    """
    return CNSP(*args, **kwargs)

def NLSP(*args, **kwargs):
    """
    Solving systems of n non-linear equations with n variables
    Parameters and usage: same as NLP
    (see help(NLP) and /examples/nlsp_*.py)
    Solvers available for now:
        scipy_fsolve (can handle df);
        converter to NLP. Example: r = p.solve('nlp:ipopt');
        nssolve (primarily for non-smooth and noisy funcs; can handle all types of constraints and 1st derivatives df,dc,dh; splitting equations to Python list or tuple is recommended to speedup calculations)
    (these ones below are very unstable and can't use user-supplied gradient - at least, for scipy 0.6.0)
        scipy_anderson
        scipy_anderson2
        scipy_broyden1
        scipy_broyden2
        scipy_broyden3
        scipy_broyden_generalized
    """
    r = CNLSP(*args, **kwargs)
    r.pWarn('''
    OpenOpt NLSP class had been renamed to SNLE 
    (system of nonlinear equations), use "SNLE" instead of "NLSP"
    ''')
    return r
    
def SNLE(*args, **kwargs):
    """
    Solving systems of n non-linear equations with n variables
    Parameters and usage: same as NLP
    (see help(NLP) and /examples/nlsp_*.py)
    Solvers available for now:
        scipy_fsolve (can handle df);
        converter to NLP. Example: r = p.solve('nlp:ipopt');
        nssolve (primarily for non-smooth and noisy funcs; can handle all types of constraints and 1st derivatives df,dc,dh; splitting equations to Python list or tuple is recommended to speedup calculations)
    (these ones below are very unstable and can't use user-supplied gradient - at least, for scipy 0.6.0)
        scipy_anderson
        scipy_anderson2
        scipy_broyden1
        scipy_broyden2
        scipy_broyden3
        scipy_broyden_generalized
    """
    return CNLSP(*args, **kwargs)

def NLLSP(*args, **kwargs):
    """
    Given set of non-linear equations
        f1(x)=0, f2(x)=0, ... fm(x)=0
    search for x: f1(x, <optional params>)^2 + ,,, + fm(x, <optional params>)^2 -> min

    Parameters and usage: same as NLP
    (see help(openopt.NLP) and /examples/nllsp_*.py)
    Solvers available for now:
        scipy_leastsq (requires scipy installed)
        converter to NLP. Example: r = p.solve('nlp:ralg')
    """
    return CNLLSP(*args, **kwargs)
    
def MOP(*args, **kwargs):
    '''
    Multiobjective optimization
    Search for weak or strong Pareto front
    
    Solvers available for now:
        interalg (http://openopt.org/interalg)
    '''
    return CMOP(*args, **kwargs)

def IP(*args, **kwargs):
    """
    Integrate a function f: R^n -> R over a given domain lb_i <= x_i <= ub_i
    """
    return CIP(*args, **kwargs)

def ODE(*args, **kwargs):
    """
    Solve ODE dy/dt = f(y,t), y(0) = y0
    """
    return CODE(*args, **kwargs)

def SLE(*args, **kwargs):
    """
    SLE: constructor for system of linear equations C*x = d assignment
    
    Examples of valid usage:
    p = SLE(C, d, <params as kwargs>)
    p = SLE(C=C, d=d, <params as kwargs>)
    """
    return CSLE(*args, **kwargs)


def DFP(*args, **kwargs):
    """
    Data Fit Problem constructor
    Search for x: Sum_i || F(x, X_i) - Y_i ||^2 -> min
    subjected to
    c(x) <= 0
    h(x) = 0
    A x <= b
    Aeq x = beq
    lb <= x <= ub
    
    Some examples of valid usage:
    p = NLP(f, x0, X, Y, <params as kwargs>)
    p = NLP(f=objFun, x0 = my_x0, X = my_X, Y=my_Y, <params as kwargs>)
    p = NLP(f, x0, X, Y, A=A, Aeq=Aeq, b=b, beq=beq, lb=lb, ub=ub, <params as kwargs>)
    Parameters and usage: same as NLP, see help(openopt.NLP)
    See also: /examples/dfp_*.py
        
    Solvers available for now:
        converter to NLP. Example: r = p.solve('nlp:ralg')
    """
    return CDFP(*args, **kwargs)


def GLP(*args, **kwargs):
    """
    GLP: constructor for general GLobal Problem 
    search for global optimum of general non-linear (maybe discontinious) function
    f(x) -> min/max
    subjected to 
    lb <= x <= ub
    Ax <= b
    c(x) <= 0
    
    usage:
    p = GLP(f, <params as kwargs>)
    
    Solving of NLPs is performed via
    r = p.solve(string_name_of_solver)
    or  p.maximize, p.minimize
    
    Parameters and usage: same as NLP  (see help(NLP) and /examples/glp_*.py)
    One more stop criterion is maxNonSuccess (default: 15)
    See also: /examples/glp_*.py

    Solvers available:
        galileo - a GA-based solver by Donald Goodman, requires finite lb <= x <= ub
        pswarm (requires PSwarm installed), license: BSD, can handle Ax<=b, requires finite search area
        de (this is temporary name, will be changed till next OO release v. 0.22), license: BSD, requires finite lb <= x <= ub, can handle Ax<=b, c(x) <= 0. The solver is based on differential evolution and made by Stepan Hlushak.
        stogo and mlsl - can use derivatives; require http://openopt.org/nlopt installed
        isres - can handle any constraints; requires http://openopt.org/nlopt installed
        interalg - exact optimum wrt required tolerance, see http://openopt.org/interalg for details
    """
    return CGLP(*args, **kwargs)


def LLSP(*args, **kwargs):
    """
    LLSP: constructor for Linear Least Squares Problem assignment
    0.5*||C*x-d||^2 + 0.5*damp*||x-X||^2 + <f,x> -> min

    subjected to:
    lb <= x <= ub

    Examples of valid calls:
    p = LLSP(C, d, <params as kwargs>)
    p = LLSP(C=my_C, d=my_d, <params as kwargs>)

    p = LLSP(C, d, lb=lb, ub=ub)

    See also: /examples/llsp_*.py

    :Parameters:
    C - float m x n numpy.ndarray, numpy.matrix or Python list of lists
    d - float array of length m (numpy.ndarray, numpy.matrix, Python list or tuple)
    damp - non-negative float number
    X - float array of length n (by default all-zeros)
    f - float array of length n (by default all-zeros)
    lb, ub - float arrays of length n (numpy.ndarray, numpy.matrix, Python list or tuple)

    :Returns:
    OpenOpt LLSP class instance

    Notes
    -----
    Solving of LLSPs is performed via
    r = p.solve(string_name_of_solver)
    r.xf - desired solution (NaNs if a problem occured)
    r.ff - objFun value (NaN if a problem occured)
    (see also other r fields)
    Solvers available for now:
    lsqr (license: GPL) - most efficient, can hanlde scipy.sparse matrices, 
        user-supplied or generated by FuncDesigner models automatically
    lapack_dgelss (license: BSD) - slow but stable, requires scipy; unconstrained
    lapack_sgelss (license: BSD) - single precesion, requires scipy; unconstrained
    bvls (license: BSD) - requires installation from OO LLSP webpage, can handle lb, ub
    converter to nlp. Example: r = p.solve('nlp:ralg', plot=1, iprint =15, <...>)
    """
    return CLLSP(*args, **kwargs)

def MMP(*args, **kwargs):
    """
    MMP: constructor for Mini-Max Problem
    search for minimum of max(func0(x), func1(x), ... funcN(x))
    See also: /examples/mmp_*.py

    Parameters and usage: same as NLP  (see help(NLP) and /examples/mmp_*.py)
    Solvers available:
        nsmm (currently unconstrained, NonSmooth-based MiniMax, uses NSP ralg solver)
    """
    return CMMP(*args, **kwargs)

def LLAVP(*args, **kwargs):
    """
   LLAVP : constructor for Linear Least Absolute Value Problem assignment
    ||C * x - d||_1  + damp*||x-X||_1-> min

    subjected to:
    lb <= x <= ub

    Examples of valid calls:
    p = LLAVP(C, d, <params as kwargs>)
    p = LLAVP(C=my_C, d=my_d, <params as kwargs>)

    p = LLAVP(C, d, lb=lb, ub=ub)

    See also: /examples/llavp_*.py

    :Parameters:
    C - float m x n numpy.ndarray, numpy.matrix or Python list of lists
    d - float array of length m (numpy.ndarray, numpy.matrix, Python list or tuple)
    damp - non-negative float number
    X - float array of length n (by default all-zeros)
    lb, ub - float arrays of length n (numpy.ndarray, numpy.matrix, Python list or tuple)

    :Returns:
    OpenOpt LLAVP class instance

    Notes
    -----
    Solving of LLAVPs is performed via
    r = p.solve(string_name_of_solver)
    r.xf - desired solution (NaNs if a problem occured)
    r.ff - objFun value (NaN if a problem occured)
    (see also other r fields)
    Solvers available for now:
    nsp:<NSP_solver_name> - converter llavp2nsp. Example: r = p.solve('nsp:ralg', plot=1, iprint =15, <...>)
    """
    return CLLAVP(*args, **kwargs)


def LUNP(*args, **kwargs):
    """
   LUNP : constructor for Linear Uniform Norm Problem assignment
    || C * x - d ||_inf (that is max | C * x - d |)  -> min

    subjected to:
    lb <= x <= ub
    A x <= b
    Aeq x = beq

    Examples of valid calls:
    p = LUNP(C, d, <params as kwargs>)
    p = LUNP(C=my_C, d=my_d, <params as kwargs>)

    p = LUNP(C, d, lb=lb, ub=ub, A = A, b = b, Aeq = Aeq, beq=beq, ...)

    See also: /examples/lunp_*.py

    :Parameters:
    C - float m x n numpy.ndarray, numpy.matrix or Python list of lists
    d - float array of length m (numpy.ndarray, numpy.matrix, Python list or tuple)
    damp - non-negative float number
    lb, ub - float arrays of length n (numpy.ndarray, numpy.matrix, Python list or tuple)

    :Returns:
    OpenOpt LUNP class instance

    Notes
    -----
    Solving of LUNPs is performed via
    r = p.solve(string_name_of_solver)
    r.xf - desired solution (NaNs if a problem occured)
    r.ff - objFun value (NaN if a problem occured)
    (see also other r fields)
    Solvers available for now:
    lp:<LP_solver_name> - converter lunp2lp. Example: r = p.solve('lp:lpSolve', <...>)
    """
    return CLUNP(*args, **kwargs)



#! /usr/bin/env python

#from .ooVersionNumber import __version__

import os, sys
curr_dir = ''.join([elem + os.sep for elem in __file__.split(os.sep)[:-1]])
sys.path += [curr_dir, curr_dir + 'kernel']

from ooVersionNumber import __version__
from oo import *

#from kernel.GUI import manage
#from kernel.oologfcn import OpenOptException
#from kernel.nonOptMisc import oosolver

from GUI import manage
from oologfcn import OpenOptException
from nonOptMisc import oosolver
from mfa import MFA


isE = False
try:
    import enthought
    isE = True
except ImportError:
    pass
try:
    import envisage
    import mayavi
    isE = True
except ImportError:
    pass
try:
    import xy
    isE = False
except ImportError:
    pass
  
if isE:
    s = """
    Seems like you are using OpenOpt from 
    commercial Enthought Python Distribution;
    consider using free GPL-licensed alternatives
    PythonXY (http://www.pythonxy.com) or
    Sage (http://sagemath.org) instead.
    """
    print(s)

    
#__all__ = filter(lambda s:not s.startswith('_'),dir())

#from numpy.testing import NumpyTest
#test = NumpyTest().test





"""
Example of MINLP
It is recommended to read help(NLP) before
and /examples/nlp_1.py 
"""
from openopt import MINLP
from numpy import *
N = 150
K = 50

#objective function:
f = lambda x: ((x-5.45)**2).sum()

#optional: 1st derivatives
df = lambda x: 2*(x-5.45)

# start point
x0 = 8*cos(arange(N))

# assign prob:
# 1st arg - objective function
# 2nd arg - start point
# for more details see 
# http://openopt.org/Assignment 
p = MINLP(f, x0, df=df, maxIter = 1e3)

# optional: set some box constraints lb <= x <= ub
p.lb = [-6.5]*N
p.ub = [6.5]*N
# see help(NLP) for handling of other constraints: 
# Ax<=b, Aeq x = beq, c(x) <= 0, h(x) = 0
# see also /examples/nlp_1.py

# required tolerance for smooth constraints, default 1e-6
p.contol = 1.1e-6

p.name = 'minlp_1'

# required field: nlpSolver - should be capable of handling box-bounds at least
#nlpSolver = 'ralg' 
nlpSolver = 'ipopt'

# coords of discrete variables and sets of allowed values
p.discreteVars = {7:range(3, 10), 8:range(3, 10), 9:[2, 3.1, 9]}

# required tolerance for discrete variables, default 10^-5
p.discrtol = 1.1e-5

#optional: check derivatives, you could use p.checkdc(), p.checkdh() for constraints
#p.checkdf()

# optional: maxTime, maxCPUTime
# p.maxTime = 15
# p.maxCPUTime = 15

r = p.solve('branb', nlpSolver=nlpSolver, plot = False)
# optim point and value are r.xf and r.ff,
# see http://openopt.org/OOFrameworkDoc#Result_structure for more details



#from numpy import *
import numpy as np
from openopt.kernel.baseSolver import baseSolver
from openopt.kernel.nonOptMisc import Vstack, Find, isspmatrix
from knitro import *
from openopt.kernel.setDefaultIterFuncs import SMALL_DELTA_X,  SMALL_DELTA_F
    
class knitro(baseSolver):
    __name__ = 'knitro'
    __license__ = "proprietary"
    __authors__ = 'Ziena Optimization LLC'
    __alg__ = ""
    __homepage__ = 'http://www.ziena.com'
    __info__ = ""
    __optionalDataThatCanBeHandled__ = ['A', 'Aeq', 'b', 'beq', 'lb', 'ub', 'c', 'h']
    _canHandleScipySparse = True
    useStopByException = False
    iterfcnConnected = True
    # CHECK ME!
    #__isIterPointAlwaysFeasible__ = lambda self, p: p.__isNoMoreThanBoxBounded__()
    
    options = ''

    def __init__(self): pass
    def __solver__(self, p):
#        try:
#            os.close(1); os.close(2) # may not work for non-Unix OS
#        except:
#            pass
        p.kernelIterFuncs.pop(SMALL_DELTA_X, None)
        p.kernelIterFuncs.pop(SMALL_DELTA_F, None)
        
        n = p.n

        ncon = p.nc + p.nh + p.b.size + p.beq.size

        g_L, g_U = np.zeros(ncon), np.zeros(ncon)
        g_L[:p.nc] = -np.inf
        g_L[p.nc+p.nh:p.nc+p.nh+p.b.size] = -np.inf

        
        # non-linear constraints, both eq and ineq
        if p.isFDmodel:
            r = []
            if p.nc != 0: r.append(p._getPattern(p.user.c))
            if p.nh != 0: r.append(p._getPattern(p.user.h))
            if p.nb != 0: r.append(p.A)
            if p.nbeq != 0: r.append(p.Aeq)
            if len(r)>0:
#                if all([isinstance(elem, np.ndarray) for elem in r]):
                r = Vstack(r)
#                else:
#                    r = Vstack(r)
#                    if isspmatrix(r):
#                        from scipy import __version__
#                        if __version__.startswith('0.7.3') or __version__.startswith('0.7.2') or __version__.startswith('0.7.1') or __version__.startswith('0.7.0'):
#                            p.pWarn('updating scipy to version >= 0.7.4 is very recommended for the problem with the solver %s' % self.__name__)
            else:
                r = np.array([])
            
            if isspmatrix(r):
                I, J, _ = Find(r)
                # DON'T remove it!
                I, J = np.array(I, np.int64), np.array(J, np.int64)
            
            elif isinstance(r, np.ndarray):
                if r.size == 0:
                    I, J= np.array([], dtype=np.int64),np.array([], dtype=np.int64)
                else:
                    I, J = np.where(r)
            
            else:
                p.disp('unimplemented type:%s' % str(type(r))) # dense matrix? 
#            nnzj = len(I)
        else:
            I, J = np.where(np.ones((ncon, p.n)))
            #I, J = None, None
#            nnzj = ncon * p.n #TODO: reduce it

        def eval_cons(x):
            r = np.array(())
            if p.userProvided.c: r = p.c(x)
            if p.userProvided.h: r = np.hstack((r, p.h(x)))
            r = np.hstack((r, p._get_AX_Less_B_residuals(x), p._get_AeqX_eq_Beq_residuals(x)))
            return r
            
        def evaluateFC (x, c):
            x = np.array(x)#TODO: REMOVE IT IN FUTURE VERSIONS
            obj = p.f(x)
            c[:] = eval_cons(x)
            return obj
    
        def eval_jac_g(x):
            r = []
            if p.userProvided.c: r.append(p.dc(x))
            if p.userProvided.h: r.append(p.dh(x))
            if p.nb != 0: r.append(p.A)
            if p.nbeq != 0: r.append(p.Aeq)
            # TODO: fix it!
#            if any([isspmatrix(elem) for elem in r]):
#                r = Vstack([(atleast_2d(elem) if elem.ndim < 2 else elem) for elem in r])
#            elif len(r)!=0:
#                r = vstack(r)
            r = Vstack(r)
            
            # TODO: make it more properly
            rr = (r.tocsr() if isspmatrix(r) else r)
            R = rr[I, J] if np.prod(rr.shape) != 0 else np.array([])
            if isspmatrix(R) or type(R) == np.matrix: 
                R = R.A
            elif not isinstance(R, np.ndarray): 
                p.err('bug in OpenOpt-knitro connection, inform OpenOpt developers, type(R) = %s' % type(R))
            return R.flatten()

        def evaluateGA(x, objGrad, jac):
            x = np.array(x)#TODO: REMOVE IT IN FUTURE VERSIONS
            df = p.df(x)
            dcons = eval_jac_g(x).tolist()
            objGrad[:] = df
            jac[:] = dcons


        objGoal = KTR_OBJGOAL_MINIMIZE
        objType = KTR_OBJTYPE_GENERAL;
        bndsLo = p.lb.copy()
        bndsUp = p.ub.copy()
        bndsLo[bndsLo < -KTR_INFBOUND] = -KTR_INFBOUND
        bndsUp[bndsUp > KTR_INFBOUND] = KTR_INFBOUND
        bndsLo = bndsLo.tolist()
        bndsUp = bndsUp.tolist()
        m = p.nc + p.nh + p.nb + p.nbeq
        cType = [ KTR_CONTYPE_GENERAL ] * (p.nc+p.nh) + [ KTR_CONTYPE_LINEAR ] * (p.nb+p.nbeq)
        cBndsLo = np.array([-np.inf]*p.nc + [0.0]*p.nh + [-np.inf]*p.nb + [0.0] * p.nbeq)
        cBndsLo[cBndsLo < -KTR_INFBOUND] = -KTR_INFBOUND
        cBndsLo = cBndsLo.tolist()
        cBndsUp = [0.0]* m
        jacIxConstr = I.tolist()
        jacIxVar    = J.tolist()
        hessRow = None#[  ]
        hessCol = None#[  ]

        xInit = p.x0.tolist()

        #---- CREATE A NEW KNITRO SOLVER INSTANCE.
        kc = KTR_new()
        if kc == None:
            raise RuntimeError ("Failed to find a Ziena license.")
            
        if KTR_set_int_param_by_name(kc, "hessopt", 4):
            raise RuntimeError ("Error setting knitro parameter 'hessopt'")
        
        if KTR_set_double_param_by_name(kc, "feastol", p.contol):
            raise RuntimeError ("Error setting knitro parameter 'feastol'")
            
        if KTR_set_char_param_by_name(kc, "outlev", "0"):
            raise RuntimeError ("Error setting parameter 'outlev'")

        if KTR_set_int_param_by_name(kc, "maxit", p.maxIter):
            raise RuntimeError ("Error setting parameter 'maxit'")

        #---- INITIALIZE KNITRO WITH THE PROBLEM DEFINITION.
        ret = KTR_init_problem (kc, n, objGoal, objType, bndsLo, bndsUp,
                                        cType, cBndsLo, cBndsUp,
                                        jacIxVar, jacIxConstr,
                                        hessRow, hessCol,
                                        xInit, None)
        if ret:
            raise RuntimeError ("Error initializing the problem, KNITRO status = %d" % ret)
        
        
        #------------------------------------------------------------------ 
        #     FUNCTION callbackEvalFC
        #------------------------------------------------------------------
         ## The signature of this function matches KTR_callback in knitro.h.
         #  Only "obj" and "c" are modified.
         ##
        def callbackEvalFC (evalRequestCode, n, m, nnzJ, nnzH, x, lambda_, obj, c, objGrad, jac, hessian, hessVector, userParams):
            if evalRequestCode == KTR_RC_EVALFC:
                obj[0] = evaluateFC(x, c)
                return 0
            else:
                return KTR_RC_CALLBACK_ERR

        #------------------------------------------------------------------
        #     FUNCTION callbackEvalGA
        #------------------------------------------------------------------
         ## The signature of this function matches KTR_callback in knitro.h.
         #  Only "objGrad" and "jac" are modified.
         ##
        def callbackEvalGA (evalRequestCode, n, m, nnzJ, nnzH, x, lambda_, obj, c, objGrad, jac, hessian, hessVector, userParams):
            if evalRequestCode == KTR_RC_EVALGA:
                evaluateGA(x, objGrad, jac)
                p.iterfcn(np.array(x), obj[0], np.max(np.array(c)))
                return 0 if p.istop == 0 else KTR_RC_USER_TERMINATION
            else:
                return KTR_RC_CALLBACK_ERR
        
        #---- REGISTER THE CALLBACK FUNCTIONS THAT PERFORM PROBLEM EVALUATION.
        #---- THE HESSIAN CALLBACK ONLY NEEDS TO BE REGISTERED FOR SPECIFIC
        #---- HESSIAN OPTIONS (E.G., IT IS NOT REGISTERED IF THE OPTION FOR
        #---- BFGS HESSIAN APPROXIMATIONS IS SELECTED).
        if KTR_set_func_callback(kc, callbackEvalFC):
            raise RuntimeError ("Error registering function callback.")
        if KTR_set_grad_callback(kc, callbackEvalGA):
            raise RuntimeError ("Error registering gradient callback.")
#        if KTR_set_hess_callback(kc, callbackEvalH):
#            raise RuntimeError ("Error registering hessian callback.")
        
#        def callbackProcessNode (evalRequestCode, n, m, nnzJ, nnzH, x, lambda_, obj, c, objGrad, jac, hessian, hessVector, userParams):
#            #---- THE KNITRO CONTEXT POINTER WAS PASSED IN THROUGH "userParams".
#            #kc = userParams
##            p.iterfcn(np.array(x), obj[0], np.max(np.array(c)))
#
#            #---- USER DEFINED TERMINATION EXAMPLE.
#            #---- UNCOMMENT BELOW TO FORCE TERMINATION AFTER 3 NODES.
#            #if KTR_get_mip_num_nodes (kc) == 3:
#            #    return KTR_RC_USER_TERMINATION
##            print p.istop
##            if p.istop != 0:
##                return KTR_RC_USER_TERMINATION
#            
#            return 0

        #---- REGISTER THE CALLBACK FUNCTION THAT PERFORMS SOME TASK AFTER
        #---- EACH COMPLETION OF EACH NODE IN BRANCH-AND-BOUND TREE.
        
        # for MINLP:
#        if KTR_set_mip_node_callback (kc, callbackProcessNode):
#            raise RuntimeError ("Error registering node process callback.")
        
        # for NLP:
#        if KTR_set_newpoint_callback (kc, callbackProcessNode):
#            raise RuntimeError ("Error registering newpoint callback.")        
        
#        if KTR_set_grad_callback (kc, callbackProcessNode):
#            raise RuntimeError ("Error registering grad callback.")        
            
        try:
            #---- SOLVE THE PROBLEM.
            #----
            #---- RETURN STATUS CODES ARE DEFINED IN "knitro.h" AND DESCRIBED
            #---- IN THE KNITRO MANUAL.
            x       = [0] * n
            lambda_ = [0] * (m + n)
            obj     = [0]
            nStatus = KTR_solve (kc, x, lambda_, 0, obj,
                                     None, None, None, None, None, None)
            x = np.array(x)
            if p.point(p.xk).betterThan(p.point(x)):
                obj = p.fk
                p.xk = p.xk.copy() # for more safety
            else:
                p.xk, p.fk = x.copy(), obj
            if p.istop == 0: p.istop = 1000
        finally:
            #---- BE CERTAIN THE NATIVE OBJECT INSTANCE IS DESTROYED.
            KTR_free(kc)





__docformat__ = "restructuredtext en"

from numpy import diag, ones, inf, any, copy, sqrt, vstack, concatenate, asarray, nan, where, array, zeros, exp, isfinite
from openopt.kernel.baseSolver import *
from openopt import LP, QP, NLP, LLSP, NSP

from openopt.kernel.ooMisc import WholeRepr2LinConst
#from scipy.optimize import line_search as scipy_optimize_linesearch
#from scipy.optimize.linesearch import line_search as scipy_optimize_linesearch_f
from numpy import arange, sign, hstack
from UkrOptMisc import getDirectionOptimPoint, getConstrDirection
import os

class lincher(baseSolver):
    __name__ = 'lincher'
    __license__ = "BSD"
    __authors__ = "Dmitrey"
    __alg__ = "a linearization-based solver written in Cherkassy town, Ukraine"
    __optionalDataThatCanBeHandled__ = ['A', 'Aeq', 'b', 'beq', 'lb', 'ub', 'c', 'h']

    __isIterPointAlwaysFeasible__ = lambda self, p: p.__isNoMoreThanBoxBounded__()
    iterfcnConnected = True

    def __init__(self): pass

    def __solver__(self, p):
        n = p.n
        x0 = copy(p.x0)
        xPrev = x0.copy()
        xf = x0.copy()
        xk = x0.copy()
        p.xk = x0.copy()

        f0 = p.f(x0)

        fk = f0
        ff = f0
        p.fk = fk

        df0 = p.df(x0)

        #####################################################################


##        #handling box-bounded problems
##        if p.__isNoMoreThanBoxBounded__():
##            for k in range(int(p.maxIter)):
##
##        #end of handling box-bounded problems
        isBB = p.__isNoMoreThanBoxBounded__()
##        isBB = 0
        H = diag(ones(p.n))
        if not p.userProvided.c:
            p.c = lambda x : array([])
            p.dc = lambda x : array([]).reshape(0, p.n)
        if not p.userProvided.h:
            p.h = lambda x : array([])
            p.dh = lambda x : array([]).reshape(0, p.n)

        p.use_subproblem = 'QP'

        #p.use_subproblem = 'LLSP'

        for k in range(p.maxIter+4):
            if isBB:
                f0 = p.f(xk)
                df = p.df(xk)
                direction = -df
                f1 = p.f(xk+direction)
                ind_l = direction<=p.lb-xk
                direction[ind_l] = (p.lb-xk)[ind_l]
                ind_u = direction>=p.ub-xk
                direction[ind_u] = (p.ub-xk)[ind_u]
                ff = p.f(xk + direction)
##                print 'f0', f0, 'f1', f1, 'ff', ff
            else:

                mr = p.getMaxResidual(xk)
                if mr > p.contol: mr_grad = p.getMaxConstrGradient(xk)
                lb = p.lb - xk #- p.contol/2
                ub = p.ub - xk #+ p.contol/2
                c, dc, h, dh, df = p.c(xk), p.dc(xk), p.h(xk), p.dh(xk), p.df(xk)
                A, Aeq = vstack((dc, p.A)), vstack((dh, p.Aeq))
                b = concatenate((-c, p.b-p.matmult(p.A,xk))) #+ p.contol/2
                beq = concatenate((-h, p.beq-p.matmult(p.Aeq,xk)))

                if b.size != 0:
                    isFinite = isfinite(b)
                    ind = where(isFinite)[0]
                    A, b = A[ind], b[ind]
                if beq.size != 0:
                    isFinite = isfinite(beq)
                    ind = where(isFinite)[0]
                    Aeq, beq = Aeq[ind], beq[ind]


                if p.use_subproblem == 'LP': #linear
                    linprob = LP(df, A=A, Aeq=Aeq, b=b, beq=beq, lb=lb, ub=ub)
                    linprob.iprint = -1
                    r2 = linprob.solve('cvxopt_glpk') # TODO: replace lpSolve by autoselect
                    if r2.istop <= 0:
                        p.istop = -12
                        p.msg = "failed to solve LP subproblem"
                        return
                elif p.use_subproblem == 'QP': #quadratic
                    qp = QP(H=H,f=df, A=A, Aeq=Aeq, b=b, beq=beq, lb=lb, ub = ub)
                    qp.iprint = -1
                    r2 = qp.solve('cvxopt_qp') # TODO: replace solver by autoselect
                    #r2 = qp.solve('qld') # TODO: replace solver by autoselect
                    if r2.istop <= 0:
                        for i in range(4):
                            if p.debug: p.warn("iter " + str(k) + ": attempt Num " + str(i) + " to solve QP subproblem has failed")
                            #qp.f += 2*N*sum(qp.A,0)
                            A2 = vstack((A, Aeq, -Aeq))
                            b2 = concatenate((b, beq, -beq)) + pow(10,i)*p.contol
                            qp = QP(H=H,f=df, A=A2, b=b2, iprint = -5)
                            qp.lb = lb - pow(10,i)*p.contol
                            qp.ub = ub + pow(10,i)*p.contol
                            # I guess lb and ub don't matter here
                            try:
                                r2 = qp.solve('cvxopt_qp') # TODO: replace solver by autoselect
                            except:
                                r2.istop = -11
                            if r2.istop > 0: break
                        if r2.istop <= 0:
                            p.istop = -11
                            p.msg = "failed to solve QP subproblem"
                            return
                elif p.use_subproblem == 'LLSP':
                    direction_c = getConstrDirection(p,  xk, regularization = 1e-7)
                else: p.err('incorrect or unknown subproblem')


            if isBB:
                X0 = xk.copy()
                N = 0
                result, newX = chLineSearch(p, X0, direction, N, isBB)
            elif p.use_subproblem != 'LLSP':
                duals = r2.duals
                N = 1.05*abs(duals).sum()
                direction = r2.xf
                X0 = xk.copy()
                result, newX = chLineSearch(p, X0, direction, N, isBB)
            else: # case LLSP
                direction_f = -df
                p2 = NSP(LLSsubprobF, [0.8, 0.8], ftol=0, gtol=0, xtol = 1e-5, iprint = -1)
                p2.args.f =  (xk, direction_f, direction_c, p, 1e20)
                r_subprob = p2.solve('ralg')
                alpha = r_subprob.xf
                newX = xk + alpha[0]*direction_f + alpha[1]*direction_c

#                dw = (direction_f * direction_c).sum()
#                cos_phi = dw/p.norm(direction_f)/p.norm(direction_c)
#                res_0, res_1 = p.getMaxResidual(xk), p.getMaxResidual(xk+1e-1*direction_c)
#                print cos_phi, res_0-res_1

#                res_0 = p.getMaxResidual(xk)
#                optimConstrPoint = getDirectionOptimPoint(p, p.getMaxResidual, xk, direction_c)
#                res_1 = p.getMaxResidual(optimConstrPoint)
#
#                maxConstrLimit = p.contol



                #xk = getDirectionOptimPoint(p, p.f, optimConstrPoint, -optimConstrPoint+xk+direction_f, maxConstrLimit = maxConstrLimit)
                #print 'res_0', res_0, 'res_1', res_1, 'res_2', p.getMaxResidual(xk)
                #xk = getDirectionOptimPoint(p, p.f, xk, direction_f, maxConstrLimit)
                #newX = xk.copy()
                result = 0
#                x_0 = X0.copy()
#                N = j = 0
#                while p.getMaxResidual(x_0) > Residual0 + 0.1*p.contol:
#                    j += 1
#                    x_0 = xk + 0.75**j * (X0-xk)
#                X0 = x_0
#                result, newX = 0, X0
#                print 'newIterResidual = ', p.getMaxResidual(x_0)

            if result != 0:
                p.istop = result
                p.xf = newX
                return

            xk = newX.copy()
            fk = p.f(xk)

            p.xk, p.fk = copy(xk), copy(fk)
            #p._df = p.df(xk)
            ####################
            p.iterfcn()

            if p.istop:
                p.xf = xk
                p.ff = fk
                #p._df = g FIXME: implement me
                return


class lineSearchFunction(object):
    def __init__(self, p, x0, N):
        self.p = p
        self.x0 = x0
        self.N = N

    def __call__(self, x):
        return float(self.p.f(x)+self.N*max(self.p.getMaxResidual(x), 0.999*self.p.contol))

    def gradient_numerical(self, x):
        g = zeros(self.p.n)
        f0 = self.__call__(x)
        for i in range(self.p.n):
            x[i] += self.p.diffInt
            g[i] = self.__call__(x) - f0
            x[i] -= self.p.diffInt
        g /= self.p.diffInt
        return g

    def gradient(self, x):
        N = self.N
        g = self.p.df(x) + N * self.p.getMaxConstrGradient(x)
        return g


def LLSsubprobF(alpha, x, direction_f, direction_c, p, S=1e30):
    x2 = x + alpha[0] * direction_f + alpha[1] * direction_c
    constr = p.getMaxResidual(x2)
    fval = p.f(x2)
    return max(constr-p.contol, 0)*S + fval
#    if constr > p.contol: return S * constr
#    else: return p.f(x2)



def chLineSearch(p, x0, direction, N, isBB):
    lsF = lineSearchFunction(p, x0, N)
    c1, c2 = 1e-4, 0.9
    result = 0
    #ls_solver = 'scipy.optimize.line_search'
    #ls_solver = 'Matthieu.optimizers.StrongWolfePowellRule'
    #ls_solver = 'Matthieu.optimizers.BacktrackingSearch'
    ls_solver = 'Armijo_modified'
##    if p.use_subproblem == 'LLSP':
##        ls_solver = 'Armijo_modified3'
##    else:
##        ls_solver = 'Armijo_modified'


    #debug
##    M, K = 1000000, 4
##    x0 = array(1.)
##    class example():
##        def __init__(self): pass
##        #def __call__(self, x): return M * max(x,array(0.))**K
##        def __call__(self, x): return 1e-5*x
##        def gradient(self, x): return array(1e-5)
##        #def gradient(self, x): return M*K*max(x,array(0.))**(K-1)
##    ff = example()
##    state = {'direction' : array(-5.5), 'gradient': M*K*x0**(K-1)}
##    mylinesearch = line_search.StrongWolfePowellRule(sigma = 0.001)
##    destination = mylinesearch(function = ff, origin = x0, step = array(-5.5), state = state)

    #debug end



    if ls_solver == 'scipy.optimize.line_search':#TODO: old_fval, old_old_fval

        old_fval = p.dotmult(lsF.gradient(x0), direction).sum() # just to make
        old_old_fval = old_fval / 2.0 # alpha0 from scipy line_search 1

        results = scipy_optimize_linesearch(lsF, lsF.gradient, x0, direction, lsF.gradient(x0), old_fval, old_old_fval, c1=c1, c2=c2)
        alpha = results[0]
##        results_f = scipy_optimize_linesearch_f(lsF, lsF.gradient, x0, direction, lsF.gradient(x0), old_fval, old_old_fval, c1=c1, c2=c2)
##        alpha = results_f[0]
        destination = x0+alpha*direction
    elif ls_solver == 'Matthieu.optimizers.BacktrackingSearch':
        #state = {'direction' : direction}
        state = {'direction' : direction, 'gradient': lsF.gradient(x0)}
        mylinesearch = line_search.BacktrackingSearch()
        destination = mylinesearch(function = lsF, origin = x0, step = direction, state = state)

    elif ls_solver == 'Matthieu.optimizers.StrongWolfePowellRule':
        state = {'direction' : direction, 'gradient': lsF.gradient(x0)}
        mylinesearch = line_search.StrongWolfePowellRule()
        destination = mylinesearch(function = lsF, origin = x0, step = direction, state = state)

    elif ls_solver == 'Armijo_modified3':
        alpha,  alpha_min = 1.0, 0.45*p.xtol / p.norm(direction)
        lsF_x0 = lsF(x0)
        C1 = abs(c1 * (p.norm(direction)**2).sum())
        iterValues.r0 = p.getMaxResidual(x0)
        #counter = 1
        while 1:
            print 'stage 1'
            if lsF(x0 + direction*alpha) <= lsF_x0 - alpha * C1 and p.getMaxResidual(x0 + direction*alpha) <= max(p.contol, iterValues.r0):
                assert alpha>=0
                #print counter, C1
                break
            alpha /= 2.0
            #counter += 1
            if alpha < alpha_min:
                if p.debug: p.warn('alpha less alpha_min')
                break
        if alpha == 1.0:
            print 'stage 2'
            K = 1.5
            lsF_prev = lsF_x0
            for i in range(p.maxLineSearch):
                lsF_new = lsF(x0 + K * direction*alpha)
                newConstr = p.getMaxResidual(x0 + K * direction*alpha)
                if lsF_new > lsF_prev or  newConstr > max(p.contol, iterValues.r0):
                    break
                else:
                    alpha *= K
                    lsF_prev = lsF_new
        destination = x0 + direction*alpha

    elif ls_solver == 'Armijo_modified':
        alpha,  alpha_min = 1.0, 0.15*p.xtol / p.norm(direction)
        grad_x0 = lsF.gradient(x0)
        #C1 = abs(c1 * p.dotmult(direction, grad_x0).sum())
        #if p.debug: print p.dotmult(direction, grad_x0).sum(), p.norm(direction)**2
        C1 = abs(c1 * (p.norm(direction)**2).sum())
        lsF_x0 = lsF(x0)
        #counter = 1
        while 1:
##            print 'stage 11'
##            print 'alpha', alpha, 'lsF', lsF(x0 + direction*alpha), 'f', p.f(x0 + direction*alpha), 'maxC', p.getMaxResidual(x0 + direction*alpha)
            if lsF(x0 + direction*alpha) <= lsF_x0 - alpha * C1:
                assert alpha>=0

##                print '11 out: alpha = ', alpha
                break
            alpha /= 2.0
            if alpha < alpha_min:
                if p.debug: p.warn('alpha less alpha_min')
                break

        destination = x0 + direction*alpha
        #TODO: check lb-ub here?

        if alpha == 1.0 and not isBB:
            K = 1.5
            lsF_prev = lsF_x0
            for i in range(p.maxLineSearch):
                x_new = x0 + K * direction*alpha
##                ind_u, ind_l = x_new>p.ub, x_new<p.lb
##                x_new[ind_l] = p.lb[ind_l]
##                x_new[ind_u] = p.ub[ind_u]

                lsF_new = lsF(x_new)
##                print 'stage 22'
##                print  'alpha', K*alpha, 'lsF', lsF_new, 'f', p.f(x0 + K * direction*alpha), 'maxC', p.getMaxResidual(x0 + K * direction*alpha)
                if lsF_new >= lsF_prev:# - K * alpha * C1:
##                    print '22 out: alpha = ', alpha
                    break
                else:
                    destination = x_new
                    alpha *= K
                    lsF_prev = lsF_new


    elif ls_solver == 'Armijo_modified2':
        grad_objFun_x0 = p.df(x0)
        grad_iterValues.r_x0 = p.getMaxConstrGradient(x0)
        C1_objFun = c1 * p.dotmult(direction, grad_objFun_x0).sum()
        C1_constr = c1 * p.dotmult(direction, grad_iterValues.r_x0).sum()
        f0 = p.f(x0)
        f_prev = f0
        allowedConstr_start = max(0.999*p.contol, p.getMaxResidual(x0))
        #currConstr = allowedConstr_start + 1.0
        alpha,  alpha_min = 1.0, 1e-11
        isConstrAccepted = False
        isObjFunAccepted = False

        #debug
##        if p.iter == 100:
##            pass

        while alpha >= alpha_min:
            x_new = x0 + direction*alpha
            if not isConstrAccepted:
                currConstr = p.getMaxResidual(x_new)
                if currConstr > allowedConstr_start + alpha * C1_constr:
                    #print 'case bigger:', currConstr, allowedConstr
                    #allowedConstr = max(0.999*p.contol, min(allowedConstr, currConstr))
                    alpha /= 2.0; continue
                else:
                    AcceptedConstr = max(0.999*p.contol, currConstr)
                    isConstrAccepted = True

            if not isObjFunAccepted:
                currConstr = p.getMaxResidual(x_new)
                #if currConstr > allowedConstr_start + alpha * C1_constr:#min(AcceptedConstr, 0.1*allowedConstr_start):
                #AllowedConstr2 = 1.2 * AcceptedConstr
                if currConstr > p.contol and (currConstr > 1.3*AcceptedConstr or currConstr > allowedConstr_start + alpha * C1_constr):# or currConstr > AllowedConstr2):
                    isObjFunAccepted = True
                    alpha = min(1.0, 2.0*alpha)#i.e. return prev alpha value
                    break

                f_new = p.f(x_new)
                #if f_new > f_prev:
                if f_new > f0 + alpha * C1_objFun:
                    alpha /= 2.0
                    f_prev = f_new
                    continue
                else:
                    isObjFunAccepted = True # and continue
                    break

        #print '!!!!!!!!!', alpha
        #print currConstr, allowedConstr_start, allowedConstr_start + alpha * C1_constr
##            else:
##                #print 'allowedConstr:', allowedConstr, '  currConstr:', currConstr
##                allowedConstr = max(0.999*p.contol, min(allowedConstr, currConstr))
##                print '33'
##                alpha /= 2.0
##                continue
##            elif p.f(x_new) <= f0 - alpha * C1 and currConstr <= allowedConstr:
##               # accept the alpha value
##               assert alpha>=0
##               allowedConstr = max(0.99*p.contol, currConstr)
##               break


        if p.debug and alpha < alpha_min:
            p.warn('alpha less alpha_min')

        if alpha == 1.0:
            K = 1.5
            f_prev = f0
            allowedConstr = allowedConstr_start
            for i in range(p.maxLineSearch):
                x_new = x0 + K*direction*alpha
                f_new = p.f(x_new)
                if f_new > f_prev or p.getMaxResidual(x_new) > allowedConstr:# - K * alpha * C1:
                    break
                else:
                    allowedConstr = max(0.99*p.contol, min(allowedConstr, currConstr))
                    alpha *= K
                    f_new = f_prev
        destination = x0 + direction*alpha
        #print 'alpha:', alpha

    else:
        p.error('unknown line-search optimizer')

    return result, destination



from openopt.kernel.baseSolver import baseSolver
from CVXOPT_SDP_Solver import CVXOPT_SDP_Solver

class dsdp(baseSolver):
    __name__ = 'dsdp'
    __license__ = "GPL"
    __authors__ = "Steven J. Benson and Yinyu Ye, Mathematics and Computer Science Division, Argonne National Laboratory, IL"
    __homepage__ = 'http://www-unix.mcs.anl.gov/DSDP/'
    #__alg__ = ""
    __optionalDataThatCanBeHandled__ = ['A', 'Aeq', 'b', 'beq', 'lb', 'ub', 'S', 'd']
    gaptol = 1e-5
    properTextOutput = True
    _canHandleScipySparse = True
    
    def __init__(self): pass

    def __solver__(self, p):
        from cvxopt import solvers
        solvers.options['DSDP_Monitor'] = p.iprint
        solvers.options['DSDP_MaxIts'] = p.maxIter
        solvers.options['DSDP_GapTolerance'] = self.gaptol
        
        return CVXOPT_SDP_Solver(p, 'dsdp')



#from numpy import asfarray, argmax, sign, inf, log10
from openopt.kernel.baseSolver import baseSolver
from numpy import asfarray,  inf,  atleast_1d
from openopt.kernel.setDefaultIterFuncs import SMALL_DELTA_X,  SMALL_DELTA_F

class galileo(baseSolver):
    __name__ = 'galileo'
    __license__ = "GPL"
    __authors__ = 'Donald Goodman, dgoodman-at-cs.msstate.edu, connected to OO by Dmitrey'
    __alg__ = "Genetic Algorithm, same as ga2, the C++ canonical genetic algorithm lib, also by Donald Goodman"
    iterfcnConnected = True
    __homepage__ = 'http://www.cs.msstate.edu/~dgoodman'

    __info__ = """
    requires finite lb, ub: lb <= x <= ub
    """
    __optionalDataThatCanBeHandled__ = ['lb', 'ub']
    __isIterPointAlwaysFeasible__ = lambda self, p: True

    population = 15
    crossoverRate = 1.0 # 1.0 means always
    mutationRate = 0.05 # not very often
    useInteger = False # use float by default
    _requiresFiniteBoxBounds = True
    _requiresBestPointDetection = True


    def __init__(self):pass
    def __solver__(self, p):

        p.kernelIterFuncs.pop(SMALL_DELTA_X)
        p.kernelIterFuncs.pop(SMALL_DELTA_F)
        p.ff = inf

        #create an initial population of 10 chromosomes
        P = Population(self.population)# CHECKME! is the Population size optimal?

        #use fitness as our evaluation function
        P.evalFunc = lambda x: -p.f(x) # fitness

        #minimum values the genes can take
        P.chromoMinValues = p.lb.tolist()

        #maximum values the genes can take
        P.chromoMaxValues = p.ub.tolist()

        #use integers instead of floats
        P.useInteger = self.useInteger

        #crossover
        P.crossoverRate = self.crossoverRate

        #mutate, but not very often
        P.mutationRate = self.mutationRate

        #use roulette (monte carlo) selection
        P.selectFunc = P.select_Roulette

        #use a full replacement size
        P.replacementSize = P.numChromosomes

        #use one point crossover
        P.crossoverFunc = P.crossover_OnePoint

        #use the default mutation routines
        P.mutateFunc = P.mutate_Default

        #use steady-state replacement with no duplication
        P.replaceFunc = P.replace_SteadyStateNoDuplicates

        #finish initializing the population. THIS MUST BE CALLED after settings the
        #variables above, but before actually running the GA!
        P.prepPopulation()
        #for 50 epochs

        for itn in range(p.maxIter+1):
          #evaluate each chromosomes
          P.evaluate()
          #apply selection
          P.select()
          #apply crossover
          P.crossover()
          #apply mutation
          P.mutate()
          #apply replacement
          P.replace()
          #print the best fit individual, and its fitness
          fval = asfarray(-P.maxFitness)
          if p.ff > fval:
              p.xf,  p.ff = asfarray(P.bestFitIndividual.genes), fval
          p.iterfcn(asfarray(P.bestFitIndividual.genes), fval)

          if p.istop:
              #p.xk,  p.fk = p.xf,  p.ff
              return



from random import Random

class Chromosome:
  """The Chromosome class represents a single chromosome in a population.
  A Chromosome contains some number of genes (Python objects), and can
  be treated as a list, with indices and slices and all that good stuff
  """

  def __init__(self):
    """Constructs a new Chromosome instance"""
    self.genes = []
    self.geneMaxValues = []
    self.geneMinValues = []
    self.fitness = None
    self.evalFunc = None
    self.parent = None

  def __str__(self):
    return self.genes.__str__()

  def randomInit(self, generator, intValues = 1):
    """Randomly initializes all genes within the ranges set with
    setMinValues and setMaxValues. generator should be an instance
    of Random from the random module. Doing things this way allows for
    thread safety. If intValues is set to 1, random
    integers will be created, else random floating point values will
    be generated.
    """
    #first, are the lists empty?
    minlen = len(self.geneMinValues)
    maxlen = len(self.geneMaxValues)
    if (minlen == 0) or (minlen != maxlen):
      return
    randFunc = None
    if intValues == 1:
      randFunc = generator.randint
    else:
      randFunc = generator.uniform
    self.genes = []
    for i in range(minlen):
      self.genes.append(randFunc(self.geneMinValues[i], self.geneMaxValues[i]))

    self.fitness = None

  def evaluate(self):
    """Calls evalFunc for this chromosome, and caches the fitness value
    returned. Returns None if evalFunc is not yet defined.
    """
    if self.evalFunc != None:
      self.fitness = self.evalFunc(self.genes)
      return self.fitness
    else:
      return None

  def getFitness(self):
    """Calls evaluate if there is no cached value, otherwise returns the cached
    fitness value.
    """
    if self.fitness != None:
      return self.fitness
    else:
      return self.evaluate()

  def copy(self):
    """Duplicates the chromosome.
    """
    retval = Chromosome()
    for item in self.__dict__:
      retval.__dict__[item] = self.__dict__[item]
    return retval

  def __len__(self):
    return len(self.genes)

  def __getitem__(self, key):
    retval = self.copy()
    retval.genes = [self.genes[key]]
    retval.geneMinValues = [self.geneMinValues[key]]
    retval.geneMaxValues = [self.geneMaxValues[key]]
    retval.fitness = None
    return retval

  def __setitem__(self, key, value):
    return self.genes.__setitem__(key, value)

  def __getslice__(self, i, j):
    retval = self.copy()
    retval.genes = self.genes[i:j]
    retval.geneMinValues = self.geneMinValues[i:j]
    retval.geneMaxValues = self.geneMaxValues[i:j]
    retval.fitness = None
    return retval
    return self.genes.__getslice__(i, j)

  def __contains__(self, item):
    return self.genes.__contains__(item)

  def __add__(self, other):
    retval = self.copy()
    retval.genes = self.genes + other.genes
    retval.geneMinValues = self.geneMinValues + other.geneMinValues
    retval.geneMaxValues = self.geneMaxValues + other.geneMaxValues
    retval.fitness = None
    return retval

  def __cmp__(self, other):
    s1 = self.getFitness()
    s2 = other.getFitness()
    return s1-s2

  def isIdentical(self, other):
    """If the genes in self and other are identical, returns 0
    """
    return (self.genes == other.genes)

class Population:
  """The Population class represents an entire population of a single
  generation of Chromosomes. This population is replaced with each iteration
  of the algorithm. Functions are provided for storing generations for later
  analysis or retrieval, or for reloading the population from some point.
  All of the high level functionality is in this
  class: generally speaking, you will almost never call a function from any
  of the other classes.
  """

  def __init__(self, numChromosomes):
    """Constructs a population of chromosomes, with numChromosomes as the
    size of the population. Note that prepPopulation must also be called
    after all user defined variables have been set, to finish initialization.
    """
    self.numChromosomes = numChromosomes
    self.currentGeneration = []
    self.nextGeneration = []
    self.chromoMaxValues = []
    self.chromoMinValues = []

    self.mutationRate = 0.0
    self.crossoverRate = 0.0
    self.replacementSize = 0
    self.useInteger = 0
    self.isSorted = 0

    self.crossoverCount = 0
    self.mutationCount = 0

    self.evalFunc = None
    self.mutateFunc = None
    self.selectFunc = None
    self.crossoverFunc = None
    self.replaceFunc = None

    self.generator = Random()

    self.minFitness = None
    self.maxFitness = None
    self.avgFitness = None
    self.sumFitness = None
    self.bestFitIndividual = None

  def prepPopulation(self):
    """Radnomly initializes each chromosome according to the values in
    chromosMinValues and chromosMaxValues.
    """
    if (len(self.chromoMinValues) != len(self.chromoMaxValues)) or (len(self.chromoMinValues) == 0):
      return None

    self.currentGeneration = []

    for i in range(self.numChromosomes):
      c = Chromosome()
      c.geneMinValues = self.chromoMinValues
      c.geneMaxValues = self.chromoMaxValues
      c.randomInit(self.generator, self.useInteger)
      c.evalFunc = self.evalFunc
      self.currentGeneration.append(c)

    return 1

  def evaluate(self):
    """Evaluates each chromosome. Since fitness values are cached, don't
    hesistate to call many times. Also calculates sumFitness, avgFitness,
    maxFitness, minFitness, and finds bestFitIndividual, for your convienence.
    Be sure to assign an evalFunc
    """

    self.sumFitness = 0.0
    self.avgFitness = 0.0
    self.maxFitness = self.currentGeneration[0].getFitness()
    self.minFitness = self.currentGeneration[0].getFitness()
    self.bestFitIndividual = self.currentGeneration[0]

    for chromo in self.currentGeneration:
      f = chromo.getFitness()
      self.sumFitness = self.sumFitness + f
      if f > self.maxFitness:
        self.maxFitness = f
        self.bestFitIndividual = chromo
      elif f < self.minFitness: self.minFitness = f

    self.avgFitness = self.sumFitness/len(self.currentGeneration)

  def mutate(self):
    """At probability mutationRate, mutates each gene of each chromosome. That
    is, each gene has a mutationRate chance of being randomly re-initialized.
    Right now, only mutate_Default is available for assignment to mutateFunc.
    """

    self.mutationCount = 0
    for i in range(self.replacementSize):
      self.nextGeneration[i] = self.mutateFunc(self.nextGeneration[i])

  def select(self):
    """Selects chromosomes from currentGeneration for placement into
    nextGeneration based on selectFunc.
    """

    self.nextGeneration = []
    for i in range(0, self.replacementSize, 2):
      s1 = self.selectFunc()
      s2 = self.selectFunc()
      s1.parent = (s1, s1)
      s2.parent = (s2, s2)
      self.nextGeneration.append(s1)
      self.nextGeneration.append(s2)

  def crossover(self):
    """Performs crossover on pairs of chromos in nextGeneration with probability
    crossoverRate. Calls crossoverFunc, which must be set; current choices are
    crossover_OnePoint, crossover_TwoPoint and crossover_Uniform.
    """

    self.crossCount = 0
    for i in range(0, self.replacementSize, 2):
      (a,b) = self.crossoverFunc(self.nextGeneration[i], self.nextGeneration[i+1])
      (self.nextGeneration[i],self.nextGeneration[i+1]) = (a,b)

  def replace(self):
    """Replaces currentGeneration with nextGeneration according to the rules
    set forth in replaceFunc. Right now, replaceFunc can take the values of
    replace_SteadyState, replace_SteadyStateNoDuplicates and
    replace_Generational.
    """

    return self.replaceFunc()

  def select_Roulette(self):
    """Perform Roulette (Monte Carlo) selection. Assign this function to
    selectFunc to use.
    In essence, we construct a big roulette wheel, with a slot for each
    individual. The size of each slot is proportional to the relative fitness
    of that individual. The wheel is then spun! whee! The more fit individuals
    have a greater chance of landing under the pointer. The individual that
    lands under the pointer is returned.
    """

    partialSum = 0.0
    #spin the wheel!!
    wheelPosition = self.generator.uniform(0, self.sumFitness)
    i = 0
    for chromo in self.currentGeneration:
      partialSum = partialSum + chromo.getFitness()
      if partialSum >= wheelPosition:
        return chromo
      i = i + 1
    return self.currentGeneration[-1]

  def select_Ranked(self):
    """Currently does nothing. Hrm.
    """
    return None

  def crossover_OnePoint(self, chromo1, chromo2):
    """A crossover function that can be assigned to crossoverFunc. This one
    takes two chromosomes, cuts them at some random point, and swaps the parts
    creating two new chromosomes, which are returned in a tuple. Note
    that there is only a crossoverRate chance of crossover happening.
    """
    prob = self.generator.random()
    if prob <= self.crossoverRate:
      self.crossoverCount = self.crossoverCount + 1
      cutPoint = self.generator.randint(0, len(chromo1)-1)
      newchromo1 = chromo1[:cutPoint]+chromo2[cutPoint:]
      newchromo2 = chromo2[:cutPoint]+chromo1[cutPoint:]
      return (newchromo1, newchromo2)
    else:
      return (chromo1, chromo2)


    """A crossover function that can be assigned to crossoverFunc. This one
    takes two chromosomes, cuts them at two random points (creating three
    parts for each chromosomes), and swaps the parts around, creating two
    new chromosomes, which are returned in a tuple. Note
    that there is only a crossoverRate chance of crossover happening.
    """
    prob = self.generator.random()
    if prob <= self.crossoverRate:
      self.crossoverCount = self.crossoverCount + 1
      cutPoint1 = self.generator.randint(0, len(chromo1)-1)
      cutPoint2 = self.generator.randint(1, len(chromo1))
      if cutPoint2 < cutPoint1:
        temp = cutPoint1
        cutPoint1 = cutPoint2
        cutPoint2 = temp

      newchromo1 = chromo1[:cutPoint1]+chromo2[cutPoint1:cutPoint2]+chromo1[cutPoint2:]
      newchromo2 = chromo2[:cutPoint1]+chromo1[cutPoint1:cutPoint2]+chromo2[cutPoint2:]

      return (newchromo1, newchromo2)
    else:
      return (chromo1, chromo2)

  def crossover_Uniform(self, chromo1, chromo2):
    """A crossover function that can be assigned to crossoverFunc. Creates
    two new chromosomes by flippinng a coin for each gene. If the coin is heads,
    the gene values in chromo1 and chromo2 are swapped (otherwise they are
    left alone). The two new chromosomes are returned in a tuple. Note
    that there is only a crossoverRate chance of crossover happening.
    """

    prob = self.generator.random()
    if prob <= self.crossoverRate:
      self.crossoverCount = self.crossoverCount + 1
      newchromo1 = chromo1.copy()
      newchromo2 = chromo2.copy()
      for i in range(len(chromo1)):
        #flip a coin...1 we switch, 0 we do nothing
        coin = self.generator.randint(0,1)
        if coin == 1:
          temp = newchromo1.genes[i]
          newchromo1.genes[i] = newchromo2.genes[i]
          newchromo2.genes[i] = temp
      return (newchromo1, newchromo2)
    else:
      return (chromo1, chromo2)

  def mutate_Default(self, chromo):
    """Mutation function that can be assigned to mutateFunc. For each gene
    on each chromosome, there is a mutationRate chance that it will be
    randomly re-initialized. The chromosome is returned.
    """
    for i in range(len(chromo.genes)):
      prob = self.generator.random()
      if prob <= self.mutationRate:
        #then we mutate!
        self.mutationCount = self.mutationCount + 1
        f = 0
        if self.useInteger:
          f = self.generator.randint(self.chromoMinValues[i], self.chromoMaxValues[i])
        else:
          f = self.generator.uniform(self.chromoMinValues[i], self.chromoMaxValues[i])
        chromo.genes[i] = f
    return chromo

  def replace_SteadyState(self):
    """Replacement function that can be assigned to replaceFunc. Takes the
    values in nextGeneration, sticks them into currentGeneration, sorts
    currentGeneration, and lops off enough of the least fit individuals
    to reduce the size of currentGeneration back to numChromosomes.
    """

    for chromo in self.nextGeneration:
      self.currentGeneration.append(chromo)
    self.currentGeneration.sort()
    self.currentGeneration.reverse()
    self.currentGeneration = self.currentGeneration[:self.numChromosomes]
    self.nextGeneration = []

  def replace_SteadyStateNoDuplicates(self):
    """Replacement function that can be assigned to replaceFunc. Same as
    replace_SteadyState, exccept that duplicate chromosomes are not inserted
    back into the currentGeneration.
    """
    #this one is like above, but no duplicates are allowed!
    for chromo in self.nextGeneration:
      flag = 0
      for chromo2 in self.currentGeneration:
        if chromo.isIdentical(chromo2):
          flag = 1
      if flag == 0:
        self.currentGeneration.append(chromo)
    self.currentGeneration.sort()
    self.currentGeneration.reverse()
    self.currentGeneration = self.currentGeneration[:self.numChromosomes]
    self.nextGeneration = []

  def replace_Generational(self):
    """Replacement function that can be assigned to replaceFunc. Wholesale
    replacement of currentGeneration with nextGeneration. assumes that
    replacementSize is equal to numChromosomes; otherwise, the
    currentGeneration will shrink in size to replacementSize in size.
    """
    self.currentGeneration = self.nextGeneration[:]
    self.nextGeneration = []



import cvxopt # to ensure cvxopt is installed
import cvxopt.base
from openopt.kernel.ooMisc import Len

matrix = cvxopt.base.matrix
sparse = cvxopt.base.sparse
Sparse = cvxopt.spmatrix

from numpy import asfarray, copy, array, prod
from openopt.kernel.nonOptMisc import isspmatrix

def Matrix(x):
    if x is None or (hasattr(x, 'shape') and prod(x.shape) == 0): return None
    if isspmatrix(x):
        if min(x.shape) > 1:
            from scipy.sparse import find
            I, J, values = find(x)
            return Sparse(array(values, float).tolist(), I.tolist(), J.tolist(), x.shape)
        else:
            x = x.toarray()
    
    x = asfarray(x)
    if x.ndim > 1 and x.nonzero()[0].size < 0.3*x.size: #todo: replace 0.3 by prob param
        return sparse(x.tolist()).T # without tolist currently it doesn't work
    else:  return matrix(x, tc='d')

def xBounds2cvxoptMatrix(p):
    """
    transforms lb - ub bounds into (A, x) <= b, (Aeq, x) = beq conditions
    this func is developed for those solvers that can handle lb, ub only via c(x)<=0, h(x)=0
    """

    #TODO: is reshape/flatten required in newest numpy versions?
    indLB, indUB, indEQ = \
    where(isfinite(p.lb) & ~(p.lb == p.ub))[0], \
    where(isfinite(p.ub) & ~(p.lb == p.ub))[0], \
    where(p.lb == p.ub)[0]    
    
    initLenB = Len(p.b)
    initLenBeq = Len(p.beq)
    nLB, nUB, nEQ = Len(indLB), Len(indUB), Len(indEQ)

    if  nLB>0 or nUB>0:
        A, b = copy(p.A), copy(p.b)
        p.A = zeros([Len(p.b) + nLB+nUB, p.n])
        p.b = zeros(Len(p.b) + nLB+nUB)
        p.b[:Len(b)] = b.flatten() # sometimes flatten is needed when called before runProbSolver(), from tests
        p.A[:Len(b)] = A
        for i in  range(len(indLB)):
            p.A[initLenB+i, indLB[i]] = -1
            p.b[initLenB+i] = -p.lb[indLB[i]]
        for i in  range(len(indUB)):
            p.A[initLenB+len(indLB)+i, indUB[i]] = 1
            p.b[initLenB+len(indLB)+i] = p.ub[indUB[i]]

    if nEQ>0:
        Aeq, beq = copy(p.Aeq), copy(p.beq)
        p.Aeq = zeros([Len(p.beq) + nEQ, p.n])
        p.beq = zeros(Len(p.beq) + nEQ)
        p.beq[:Len(beq)] = beq
        p.Aeq[:Len(beq)] = Aeq
        for i in range(len(indEQ)):
            p.Aeq[initLenBeq+i, indEQ[i]] = 1
            p.beq[initLenBeq+i] = p.lb[indEQ[i]] # = p.ub[indEQ[i]], because they are the same

    p.lb = -inf*ones(p.n)
    p.ub = inf*ones(p.n)



from scipy.linalg.flapack import sgelss
from numpy.linalg import norm
from numpy import dot, asfarray, atleast_1d
from openopt.kernel.baseSolver import baseSolver

class lapack_sgelss(baseSolver):
    __name__ = 'lapack_sgelss'
    __license__ = "BSD"
    __authors__ = 'Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd., Courant Institute, Argonne National Lab, and Rice University'
    #__alg__ = "SVD"
    __info__ = 'wrapper to LAPACK sgelss routine (single precision), requires scipy & LAPACK 3.0 or newer installed'
    def __init__(self):pass
    def __solver__(self, p):
        res = sgelss(p.C, p.d)
        x,info = res[1], res[-1]
        xf = x[:p.C.shape[1]]
        ff = atleast_1d(asfarray(p.F(xf)))
        p.xf = p.xk = xf
        p.ff = p.fk = ff
        if info == 0: p.istop = 1000
        else: p.istop = -1000



from openopt import oosolver, NLP
ipopt = oosolver('ipopt', color='r') # oosolver can hanlde prob parameters
ralg = oosolver('ralg', color='k', alp = 4.0) # as well as solver parameters
solvers = [ralg, ipopt]
for solver in solvers:
    assert solver.isInstalled, 'solver ' + solver.__name__ + ' is not installed'
    p = NLP(x0 = 15, f = lambda x: x**4, df = lambda x: 4 * x**3, iprint = 0)
    r = p.solve(solver, plot=1, show = solver == solvers[-1])



# Example of export OpenOpt LP to MPS file
# you should have lpsolve and its Python binding properly installed
# (you may take a look at the instructions from openopt.org/LP)

# You can solve problems defined in MPS files 
# with a variety of solvers at NEOS server for free
# http://neos.mcs.anl.gov/
# BTW they have Python API along with web API and other

from numpy import *
from openopt import LP
f = array([15,8,80])
A = mat('1 2 3; 8 15 80; 8 80 15; -100 -10 -1') # numpy.ndarray is also allowed
b = [15, 80, 150, -800] # numpy.ndarray, matrix etc are also allowed
Aeq = mat('80 8 15; 1 10 100') # numpy.ndarray is also allowed
beq = (750, 80)

lb = [4, -80, -inf]
ub = [inf, -8, inf]
p = LP(f, A=A, Aeq=Aeq, b=b, beq=beq, lb=lb, ub=ub, name = 'lp_1')
# or p = LP(f=f, A=A, Aeq=Aeq, b=b, beq=beq, lb=lb, ub=ub)

# if file name not ends with '.MPS' or '.mps'
# then '.mps' will be appended
success = p.exportToMPS('asdf') 
# success is False if a error occurred (read-only file system, no write access, etc)
# elseware success is True

# objFunValue should be 204.48841578
# x_opt should be [ 9.89355041 -8.          1.5010645 ]



from baseProblem import NonLinProblem

from numpy import asfarray, dot, abs, ndarray
import numpy as np
from setDefaultIterFuncs import FVAL_IS_ENOUGH, SMALL_DELTA_F
import NLP
try:
    import scipy
    solver = 'scipy_fsolve'
except ImportError:
    solver = 'nssolve'

class NLSP(NonLinProblem):
    _optionalData = ['A', 'Aeq', 'b', 'beq', 'lb', 'ub', 'c', 'h']
    probType = 'NLSP'
    goal = 'solution'
    allowedGoals = ['solution']
    isObjFunValueASingleNumber = False
    showGoal = False
    expectedArgs = ['f', 'x0']
    solver = solver
    
    def __init__(self, *args, **kwargs):
        NonLinProblem.__init__(self, *args, **kwargs)
        if 'is_oovar' in dir(self.f):
            self.f = [self.f]

    def objFuncMultiple2Single(self, fv):
        #return norm(atleast_1d(asfarray(fv)), inf)
        # for pypy:
        return np.max(np.abs(fv))

    def nlsp2nlp(self, solver, **solver_params):
        ff = lambda x: sum(asfarray(self.f(x))**2)
        p = NLP.NLP(ff, self.x0)
        self.fill(p, sameConstraints=True)
        if self.userProvided.df:
            p.df = lambda x: p.matmult(2*asfarray(self.f(x)), asfarray(self.df(x)))
        p.f = ff


        def nlsp_iterfcn(*args,  **kwargs):
            if len(args) != 0 and type(args[0]) != ndarray: # hence Point
                p.primalIterFcn(args[0].x, max(abs(self.f(args[0].x))), args[0].mr(),  **kwargs)
                # TODO: add nNaNs
            elif len(args) > 1:
                p.primalIterFcn(args[0], max(abs(self.f(args[0]))), *args[2:],  **kwargs)
            elif 'fk' in kwargs.keys():
                kwargs['fk'] = max(abs(self.f(args[0])))
                p.primalIterFcn(*args, **kwargs)
            else:
                p.primalIterFcn(*args,  **kwargs)
            p.xk = self.xk
            p.fk = p.f(p.xk)
            p.rk = self.rk
            # TODO: add nNaNs

            p.istop = self.istop
            return p.istop

        ftol_init = self.ftol
        contol_init = self.contol
        
        msg_contol = '' if self.isUC else 'and contol '
        
        def nlsp_callback(nlsp):
            # nlsp = self
            if all(abs(asfarray(self.f(nlsp.xk))) < ftol_init)  and (self.isUC or self.getMaxResidual(nlsp.xk) < contol_init):
                self.msg = 'solution with required ftol ' + msg_contol+ 'has been reached'
                return (15, self.msg)
            else:
                return False

        self.callback = [nlsp_callback]
        self.kernelIterFuncs.pop(SMALL_DELTA_F, None)
        p.primalIterFcn,  p.iterfcn = self.iterfcn, nlsp_iterfcn
        p.goal = 'min'
        #self.fEnough = self.ftol


        p.iprint = -1

        Multiplier = 1e16

        #self.ftol /= Multiplier
        self.xtol /= Multiplier
        self.gtol /= Multiplier

        p.show = False
        
        p.fOpt = 0.0
        p.fTol = p.ftol**2
        
        r = p.solve(solver, **solver_params)

        #self.ftol *= Multiplier
        self.xtol *= Multiplier
        self.gtol *= Multiplier

        if self.istop == FVAL_IS_ENOUGH:
            self.msg = 'solution with required ftol ' + msg_contol+ 'has been reached'
            self.istop = 15

        #self.iterfcn(xk = r.xk, fk = r.fk, rk = r.rk)
        #self.show = show

        # TODO: fix it!
        #r.iterValues.f = self.iterValues.f

        #r.ff = max(abs(asfarray(self.f(r.xf))))
        return r



import sys
sys.path.append('/home/dmitrey/OOSuite/OpenOpt')

from numpy import empty, sin, cos, arange
from openopt import LLSP

M, N = 1500, 1000
C = empty((M,N))
d =  empty(M)

for j in range(M):
    d[j] = 1.5*N+80*sin(j)
    C[j] = 8*sin(4.0+arange(N)) + 15*cos(j)
    
""" alternatively, try the sparse problem - lsqr solver can take benefits of it.
Also, if your C is too large for your RAM 
you can pass C of any scipy.sparse matrix format

for j in xrange(M):
    d[j] = 1.5*N+80*sin(j)
    C[j, j%N] = 15*cos(j) #+ 8*sin(4.0+arange(N))
    C[j, (1 + j)%N] = 15*cos(j) #+ 8*sin(4.0+arange(N))
"""

p = LLSP(C, d)
r = p.solve('lsmr')

print('f_opt: %f' % r.ff) # 2398301.68347
#print 'x_opt:', r.xf




"""
Let us solve the overdetermined nonlinear equations:
a^2 + b^2 = 15
a^4 + b^4 = 100
a = 3.5

Let us concider the problem as
x[0]**2 + x[1]**2 - 15 = 0
x[0]**4 + x[1]**4 - 100 = 0
x[0] - 3.5 = 0

Now we will solve the one using solver scipy_leastsq
"""
from openopt import NLLSP
from numpy import *

f = lambda x: ((x**2).sum() - 15, (x**4).sum() - 100, x[0]-3.5)
# other possible f assignments:
# f = lambda x: [(x**2).sum() - 15, (x**4).sum() - 100, x[0]-3.5]
#f = [lambda x: (x**2).sum() - 15, lambda x: (x**4).sum() - 100, lambda x: x[0]-3.5]
# f = (lambda x: (x**2).sum() - 15, lambda x: (x**4).sum() - 100, lambda x: x[0]-3.5)
# f = lambda x: asfarray(((x**2).sum() - 15, (x**4).sum() - 100, x[0]-3.5))
#optional: gradient
def df(x):
    r = zeros((3,2))
    r[0,0] = 2*x[0]
    r[0,1] = 2*x[1]
    r[1,0] = 4*x[0]**3
    r[1,1] = 4*x[1]**3
    r[2,0] = 1
    return r

# init esimation of solution - sometimes rather pricise one is very important
x0 = [1.5, 8]

#p = NLLSP(f, x0, diffInt = 1.5e-8, xtol = 1.5e-8, ftol = 1.5e-8)
# or
# p = NLLSP(f, x0)
# or
p = NLLSP(f, x0, df = df, xtol = 1.5e-8, ftol = 1.5e-8)

#optional: user-supplied gradient check:
p.checkdf()
#r = p.solve('scipy_leastsq', plot=1, iprint = -1)
#or using converter lsp2nlp:
r = p.solve('nlp:ralg', iprint = 1, plot=1)
#r = p.solve('nlp:ipopt',plot=1), r = p.solve('nlp:algencan'), r = p.solve('nlp:ralg'), etc
#(some NLP solvers require additional installation)

print 'x_opt:', r.xf # 2.74930862,  +/-2.5597651
print 'funcs Values:', p.f(r.xf) # [-0.888904734668, 0.0678251418575, -0.750691380965]
print 'f_opt:', r.ff, '; sum of squares (should be same value):', (p.f(r.xf) ** 2).sum() # 1.35828942657



#!/usr/bin/python
import numpy as np
import random
random.seed(15)
import sys

#from PIL import Image, ImageDraw, ImageFont
from math import sqrt

def rand_seq(size):
    '''generates values in random order
    equivalent to using shuffle in random,
    without generating all values at once'''
    values=np.arange(size).tolist()
    for i in range(size):
        # pick a random index into remaining values
        j=i+int(random.random()*(size-i))
        # swap the values
        values[j],values[i]=values[i],values[j]
        # return the swapped value
        yield values[i] 

def all_pairs(size):
    '''generates all i,j pairs for i,j from 0-size'''
    for i in rand_seq(size):
        for j in rand_seq(size):
            yield (i,j)

def reversed_sections(tour):
    '''generator to return all possible variations where the section between two cities are swapped'''
    for i,j in all_pairs(len(tour)):
        if i != j:
            copy=tour[:]
            if i < j:
                copy[i:j+1]=tour[i:j+1][::-1]
            else:
                copy[i+1:] = tour[:j][::-1]
                copy[:j] = tour[i+1:][::-1]
            if not np.array_equal(copy, tour): # no point returning the same tour
                yield copy

def swapped_cities(tour):
    '''generator to create all possible variations where two cities have been swapped'''
    for i,j in all_pairs(len(tour)):
        if i < j:
            copy=tour[:]
            copy[i],copy[j]=tour[j],tour[i]
            yield copy

def cartesian_matrix(coords):
    '''create a distance matrix for the city coords that uses straight line distance'''
    matrix={}
    for i,(x1,y1) in enumerate(coords):
        for j,(x2,y2) in enumerate(coords):
            dx,dy=x1-x2,y1-y2
            dist=sqrt(dx*dx + dy*dy)
            matrix[i,j]=dist
    return matrix

def read_coords(coord_file):
    '''
    read the coordinates from file and return the distance matrix.
    coords should be stored as comma separated floats, one x,y pair per line.
    '''
    coords=[]
    for line in coord_file:
        x,y=line.strip().split(',')
        coords.append((float(x),float(y)))
    return coords

def tour_length(matrix,tour):
    tour = np.array(tour, int)
    
    '''total up the total length of the tour based on the distance matrix'''
    total=0
    num_cities=len(tour)
    for i in range(num_cities):
        j=(i+1)%num_cities
        city_i=tour[i]
        city_j=tour[j]
        total+=matrix[city_i,city_j]
    return total




def write_tour_to_img(coords,tour,title,img_file):
    padding=20
    # shift all coords in a bit
    coords=[(x+padding,y+padding) for (x,y) in coords]
    maxx,maxy=0,0
    for x,y in coords:
        maxx=max(x,maxx)
        maxy=max(y,maxy)
    maxx+=padding
    maxy+=padding
    img=Image.new("RGB",(int(maxx),int(maxy)),color=(255,255,255))
    
    font=ImageFont.load_default()
    d=ImageDraw.Draw(img);
    num_cities=len(tour)
    for i in range(num_cities):
        j=(i+1)%num_cities
        city_i=tour[i]
        city_j=tour[j]
        x1,y1=coords[city_i]
        x2,y2=coords[city_j]
        d.line((int(x1),int(y1),int(x2),int(y2)),fill=(0,0,0))
        d.text((int(x1)+7,int(y1)-5),str(i),font=font,fill=(32,32,32))
    
    
    for x,y in coords:
        x,y=int(x),int(y)
        d.ellipse((x-5,y-5,x+5,y+5),outline=(0,0,0),fill=(196,196,196))
    
    d.text((1,1),title,font=font,fill=(0,0,0))
    
    del d
    img.save(img_file, "PNG")

def init_random_tour(tour_length):
   tour=np.arange(tour_length).tolist()
   random.shuffle(tour)
   return tour

def run_hillclimb(init_function,move_operator,objective_function,max_iterations):
    from hillclimb import hillclimb_and_restart
    iterations,score,best=hillclimb_and_restart(init_function,move_operator,objective_function,max_iterations)
    return iterations,score,best

def run_anneal(init_function,move_operator,objective_function,max_iterations,start_temp,alpha, p=None):
    if start_temp is None or alpha is None:
        usage();
        print("missing --cooling start_temp:alpha for annealing")
        sys.exit(1)
    from sa import anneal
    iterations,score,best=anneal(init_function,move_operator,objective_function,max_iterations,start_temp,alpha, p)
    return iterations,score,best

def usage():
    print("usage: python %s [-o <output image file>] [-v] [-m reversed_sections|swapped_cities] -n <max iterations> [-a hillclimb|anneal] [--cooling start_temp:alpha] <city file>" % sys.argv[0])

def main(arg, p = None):
    max_iterations=10000

    move_operator=reversed_sections
    #move_operator=swapped_cities
    
    start_temp=10
    alpha=0.99995
    #start_temp,alpha=None,None

    def run_anneal_with_temp(init_function,move_operator,objective_function,max_iterations, p=None):
        return run_anneal(init_function,move_operator,objective_function,max_iterations,start_temp,alpha, p)
    run_algorithm=run_anneal_with_temp#run_hillclimb

    
    if isinstance(arg, dict):
        matrix = arg
        tmp = (1 + sqrt(1+4*len(arg))) / 2
        lc = np.round(tmp)
        assert -0.00001 < tmp - lc < 0.00001, 'matrix seems to have incorrect size'
#        assert matrix.ndim == 2, 'matrix of distances must have dimension 2'
#        assert matrix.shape[0] == matrix.shape[1], 'square matrix is expected'
#        lc = matrix.shape[0]
    else:
        city_file = arg
        F = open(city_file)
        coords=read_coords(F)
        matrix=cartesian_matrix(coords)
        lc = len(coords)
        
    init_function=lambda: init_random_tour(lc)
#    if iterfcn is None:
    objective_function=lambda tour: -tour_length(matrix,tour)
#    else:
#        def objective_function(tour):
#            r = -tour_length(matrix,tour)
#            objective_function.iterfcn_counter += 1
#            if not objective_function.iterfcn_counter % 1024:
#                iterfcn(np.array(tour), r)
#            return r
#        objective_function.iterfcn_counter = 0
    if p is not None:
        iterations,score,best=run_algorithm(init_function,move_operator, objective_function, p.maxFunEvals+10, p)
    else:
        iterations,score,best=run_algorithm(init_function,move_operator,objective_function, max_iterations)
    # output results
    return (iterations,score,best)    

if __name__ == "__main__":
    from time import time
    t = time()
    city_file = '/home/dmitrey/tsp/part_three/city100.txt'
    r = main(city_file)
    print(r)
    print('time elapsed: %0.1f' %(time()-t))




from scipy.optimize import fminbound
from openopt.kernel.baseSolver import baseSolver

class scipy_fminbound(baseSolver):
    __name__ = 'scipy_fminbound'
    __optionalDataThatCanBeHandled__ = ['lb', 'ub']
    __license__ = "BSD"
    __authors__ = "Travis E. Oliphant"
    __alg__ = "Brent's method (golden section + parabolic fit)"
    __info__ = '1-dimensional minimizer for finite box-bound problems'
    __isIterPointAlwaysFeasible__ = lambda self, p: True
    def __init__(self):pass
    def __solver__(self, p):
        if p.n != 1: p.err('the solver ' + self.__name__ +' can handle singe-variable problems only')
        if not p.__isFiniteBoxBounded__(): p.err('the solver ' + self.__name__ +' requires finite lower and upper bounds')
        xf, ff, ierr, numfunc= fminbound(p.f, p.lb, p.ub, disp = 0, xtol = 0.999*p.xtol, maxfun = p.maxFunEvals, full_output = 1)

        p.xk = p.xf = xf
        p.fk = p.ff = ff
        if ierr==0: p.istop = 1000
        p.iterfcn()




'''
Copyright (c) 2010 Enzo Michelangeli and IT Vision Ltd

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''
from numpy import *
from qlcp import qlcp
try:
    import openopt
except:
    pass # if OpenOpt is not installed, the qpsolver kwarg can't be specified
    
def _simple_grad(f, x, delta = 1e-8):
    nvars = x.shape[0]
    Id = eye(nvars)*delta
    grad = array([(f(x+Id[i,:]) - f(x-Id[i,:]))/(2*delta) for i in range(nvars)])
    return grad

def _simple_hessian(f, x, delta = 1e-4):    # generally too slow for use
    g = lambda x: _simple_grad(f, x, delta = delta) # g(x) is the gradient of f
    return _simple_grad(g, x, delta=delta)

def _simple_hessdiag(f, x, delta = 1e-4):
    nvars = x.shape[0]
    Id = eye(nvars)*delta
    hd = array([(f(x+Id[i,:]) + f(x-Id[i,:]) - 2*f(x))/delta**2 for i in range(nvars)]).flatten()
    return diag(hd)

def sqlcp(f, x0, df=None, A=None, b=None, Aeq=None, beq=None, lb=None, ub=None, minstep=1e-15, minfchg=1e-15, qpsolver=None, callback = None):
    '''
    SQP solver. Approximates f in x0 with paraboloid with same gradient and hessian,
    then finds its minimum with a quadratic solver (qlcp by default) and uses it as new point, 
    iterating till changes in x and/or f drop below given limits. 
    Requires the Hessian to be definite positive.
    The Hessian is initially approximated by its principal diagonal, and then
    updated at every step with the BFGS method.
    f:        objective function of x to be minimized
    x0:       initial value for f
    df:       gradient of f: df(f) should return a function of such as f(x) would
              return the gradient of f in x. If missing or None, an approximation 
              will be calculated with an internal finite-differences procedure.
    A:        array of inequality constraints (A x >= b)
    b:        right-hand side of A x >= b
    Aeq:      array of equality constraints (Aeq x = beq)
    beq:      right-hand side of Aeq x >= beq
    lb:       lower bounds for x (assumed -Inf if missing)
    ub:       upper bounds for x (assumed +Inf if missing)
    minstep:  iterations terminate when updates to x become < minstep (default: 1e-15)
    minfchg:  iterations terminate when RELATIVE changes in f become < minfchg (default: 1e-15)
    qpsolver: if None, qlcp; else a solver accepted by openopt.QP (if OpenOpt and 
              that particular solver are installed)
    '''
       
    nvars = x0.shape[0]
    x = x0.copy()
    niter = 0
    deltah = 1e-4
    deltag = deltah**2
    
    if df == None:  # df(x) is the gradient of f in x
        df = lambda x: _simple_grad(f, x, deltag)
    
    twoI = 2.*eye(nvars)
    oldfx = f(x)
    gradfx = df(x)  # return the gradient of f() at x
    hessfx = _simple_hessdiag(f,x,delta=deltah) # good enough, and much faster, but only works if REAL Hessian is DP!
    invhessfx = linalg.inv(hessfx)
    while True:
        niter += 1
        
        # compute the b, beq, lb and ub for the QP sub-problem (as bx, beqx, lbx, ubx)
        bx = b if b == None else b-dot(A,x)
        beqx = beq if beq == None else beq-dot(Aeq,x)
        lbx = lb if lb == None else lb - x
        ubx = ub if ub == None else ub - x

        if qpsolver == None:
            deltax = qlcp(hessfx, gradfx, A=A, b=bx, Aeq=Aeq, beq=beqx, lb=lbx, ub=ubx, QI=invhessfx)
        else:
            p = openopt.QP(hessfx, gradfx, A=A, b=bx, Aeq=Aeq, beq=beqx, lb=lbx, ub=ubx)
            p.ftol = 1.e-10
            r = p.solve(qpsolver, iprint = -1)
            deltax = p.xf
        
        if deltax == None:
            #print("Cannot converge, sorry.")
            x = None
            break
        
        x += deltax
        if linalg.norm(deltax) < minstep:
            break
        fx = f(x)
        if abs(fx-oldfx) < minfchg*abs(fx):
            break
        if callback is not None and callback(x):
            break
            
        oldfx = fx
        oldgradfx = gradfx.copy()
        gradfx = df(x)  # return the gradient of f() at the new x
        # we might also put a termination test on the norm of grad...
        
        '''
        # recalc hessian afresh would be sloooow...
        #hessfx = _simple_hessian(f,x,delta=deltah)  # return the hessian of f() at x
        hessfx = _simple_hessdiag(f,x,delta=deltah)  # return the hessian (diag only) of f() at x
        invhessfx = linalg.inv(hessfx)
        '''
        # update Hessian and its inverse with BFGS based on current Hessian, deltax and deltagrad    
        # See http://en.wikipedia.org/wiki/BFGS
        deltagrad = gradfx - oldgradfx
        hdx = dot(hessfx, deltax)
        dgdx = dot(deltagrad,deltax)
        #if dgdx < 0.:
        #    print "deltagrad * deltax < 0!" # a bad sign
        hessfx += ( outer(deltagrad,deltagrad) / dgdx - 
                    outer(hdx, hdx) / dot(deltax, hdx) )
        # now update inverse of Hessian  
        '''
        invhessfx = linalg.inv(hessfx)
        '''
        hidg = dot(invhessfx,deltagrad)
        oIdgdeltax = outer(hidg,deltax)
        invhessfx += ( (dgdx+dot(deltagrad,hidg))*outer(deltax,deltax)/(dgdx**2) -
                (oIdgdeltax+oIdgdeltax.T)/dgdx ) # just because invhessfx is symmetric, or else:
                #(oIdgdeltax+outer(deltax,dot(invhessfx.T,deltagrad)))/dgdx )
    return x, niter
    



from baseProblem import MatrixProblem
from numpy import ones, inf, dot, zeros, hstack, vstack, atleast_2d
from ooMisc import norm
import LP

class LUNP(MatrixProblem):
    probType = 'LUNP'
    goal = 'minimum'
    allowedGoals = ['minimum', 'min']
    showGoal = False
    _optionalData = []
    expectedArgs = ['C', 'd']
    def __init__(self, *args, **kwargs):
        MatrixProblem.__init__(self, *args, **kwargs)
        self.n = self.C.shape[1]
    #    if 'damp' not in kwargs.keys(): kwargs['damp'] = None
    #    if 'X' not in kwargs.keys(): kwargs['X'] = nan*ones(self.n)

        if self.x0 is None: self.x0 = zeros(self.n)
        
        #lunp_init(self, kwargs)

    def objFunc(self, x):
        r = norm(dot(self.C, x) - self.d, inf)
#        if not self.damp is None:
#            r += self.damp * norm(x-self.X)**2 / 2.0
#        if any(isfinite(self.f)): r += dot(self.f, x)
        return r

    def lunp2lp(self, solver, **solver_params):
        shapeC = atleast_2d(self.C).shape
        nVars = shapeC[1] + 1
        nObj = shapeC[0]
        f = hstack((zeros(nVars)))
        f[-1] = 1
        p = LP.LP(f)
        # TODO: check - is it performed in self.fill(p)?
        if hasattr(self,'x0'): p.x0 = self.x0
        #p.args.f = self # DO NOT USE p.args = self IN PROB ASSIGNMENT!
        self.fill(p)
        p.x0 = hstack((p.x0, [0]))
        p.A = vstack((hstack((self.A, zeros((atleast_2d(self.A).shape[0], 1)))), \
                      hstack((self.C, -ones((nObj, 1)))), \
                      hstack((-self.C, -ones((nObj, 1))))))
        p.b = hstack((p.b, self.d, -self.d))
        
        p.lb = hstack((p.lb, -inf))
        p.ub = hstack((p.ub, inf))
        
        p.Aeq = hstack((self.Aeq, zeros((atleast_2d(self.Aeq).shape[0], 1))))
        
        #p.iprint = -1
        self.iprint = -1
        # for LLSP plot is via NLP
        #p.show = self.show
        #p.plot, self.plot = self.plot, 0
        #p.checkdf()
        r = p.solve(solver, **solver_params)
        self.xf, self.ff, self.rf = r.xf[:-1], r.ff, r.rf
        self.istop, self.msg = p.istop, p.msg
        return r

#    def _Prepare(self):
#        MatrixProblem._Prepare(self)
#        if not self.damp is None and not any(isfinite(self.X)):
#            self.X = zeros(self.n)




#def ff(x, LLSPprob):
#    r = dot(LLSPprob.C, x) - LLSPprob.d
#    return dot(r, r)
#ff = lambda x, LLSPprob: LLSPprob.objFunc(x)
#def dff(x, LLSPprob):
#    r = dot(LLSPprob.C.T, dot(LLSPprob.C,x)  - LLSPprob.d)
#    if not LLSPprob.damp is None: r += LLSPprob.damp*(x - LLSPprob.X)
#    if all(isfinite(LLSPprob.f)) : r += LLSPprob.f
#    return r
#
#def d2ff(x, LLSPprob):
#    r = dot(LLSPprob.C.T, LLSPprob.C)
#    if not LLSPprob.damp is None: r += LLSPprob.damp*eye(x.size)
#    return r



from baseProblem import NonLinProblem
#from numpy.linalg import norm
from numpy import inf

class IP(NonLinProblem):
    probType = 'IP'
    goal = 'solution'
    allowedGoals = ['solution']
    showGoal = False
    _optionalData = []
    expectedArgs = ['f', 'domain']
    ftol = None
    def __init__(self, *args, **kwargs):
        NonLinProblem.__init__(self, *args, **kwargs)
        domain = args[1]
        self.x0 = dict([(v, 0.5*(val[0]+val[1])) for v, val in domain.items()])
        self.constraints = [v>bounds[0] for v,  bounds in domain.items()] + [v<bounds[1] for v,  bounds in domain.items()]
        #self.data4TextOutput = ['objFunVal', 'residual']
        self._Residual = inf

    def objFunc(self, x):
        return 0
        #raise 'unimplemented yet'
        
        #r = norm(dot(self.C, x) - self.d) ** 2  /  2.0
        #return r



from wh_conn_solver import wh_conn_solver

class matlab_ode(wh_conn_solver):
    __license__ = "proprietary"
    __authors__ = ""
    __alg__ = ""
    __optionalDataThatCanBeHandled__ = []
    arrAttribs = ('x0', 'times')
    funcAttribs = ('f', 'df')
    solver_id = 100

    def __init__(self): 
        pass

